@@ -4,7 +4,7 @@ package bash
 
 import (
 	"fmt"
-	"github.com/jfrog/jfrog-cli-core/utils/coreutils"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/coreutils"
 	"github.com/jfrog/jfrog-client-go/utils/log"
 	"os"
 	"path/filepath"
@@ -20,9 +20,14 @@ _jfrog() {
 }
 
 complete -F _jfrog -o default jfrog
+complete -F _jfrog -o default jf
 `
 
-func WriteBashCompletionScript() {
+func WriteBashCompletionScript(install bool) {
+	if !install {
+		fmt.Print(BashAutocomplete)
+		return
+	}
 	homeDir, err := coreutils.GetJfrogHomeDir()
 	if err != nil {
 		log.Error(err)
