@@ -1,3 +1,4 @@
+//go:build ignore
 // +build ignore
 
 // This program generates bash and zsh completion scripts.
@@ -6,11 +7,11 @@ package main
 
 import (
 	"errors"
-	"github.com/jfrog/jfrog-cli-core/utils/coreutils"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/coreutils"
 	"os"
 	"strings"
 
-	"github.com/jfrog/jfrog-cli-core/utils/log"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/log"
 	"github.com/jfrog/jfrog-cli/completion/shells/bash"
 	"github.com/jfrog/jfrog-cli/completion/shells/zsh"
 )
@@ -29,11 +30,13 @@ func main() {
 }
 
 func writeScript(script string) {
-	scriptFile, err := os.Create("jfrog")
-	coreutils.ExitOnErr(err)
-	defer scriptFile.Close()
-	err = os.Chmod("jfrog", os.ModePerm)
-	coreutils.ExitOnErr(err)
-	_, err = scriptFile.WriteString(script)
-	coreutils.ExitOnErr(err)
+	for _, exe := range []string{"jfrog", "jf"} {
+		scriptFile, err := os.Create(exe)
+		coreutils.ExitOnErr(err)
+		defer scriptFile.Close()
+		err = os.Chmod(exe, os.ModePerm)
+		coreutils.ExitOnErr(err)
+		_, err = scriptFile.WriteString(script)
+		coreutils.ExitOnErr(err)
+	}
 }
