@@ -1,33 +1,42 @@
 package main
 
 import (
+	"encoding/json"
 	"errors"
-	"github.com/jfrog/jfrog-client-go/utils/log"
-	"os"
-	"path/filepath"
-	"testing"
-
-	"github.com/jfrog/jfrog-cli-core/utils/coreutils"
-
-	"github.com/jfrog/jfrog-cli-core/utils/config"
+	"fmt"
+	coreTestUtils "github.com/jfrog/jfrog-cli-core/v2/common/tests"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/config"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/coreutils"
+	coreTests "github.com/jfrog/jfrog-cli-core/v2/utils/tests"
 	"github.com/jfrog/jfrog-cli/inttestutils"
 	"github.com/jfrog/jfrog-cli/utils/tests"
 	"github.com/jfrog/jfrog-client-go/auth"
+	distributionServices "github.com/jfrog/jfrog-client-go/distribution/services"
 	clientDistUtils "github.com/jfrog/jfrog-client-go/distribution/services/utils"
-	"github.com/jfrog/jfrog-client-go/utils"
+	clientUtils "github.com/jfrog/jfrog-client-go/utils"
+	"github.com/jfrog/jfrog-client-go/utils/distribution"
 	"github.com/jfrog/jfrog-client-go/utils/io/fileutils"
 	"github.com/jfrog/jfrog-client-go/utils/io/httputils"
+	"github.com/jfrog/jfrog-client-go/utils/log"
 	"github.com/stretchr/testify/assert"
+	"net/http"
+	"os"
+	"path/filepath"
+	"strconv"
+	"testing"
 )
 
-const bundleVersion = "10"
+const (
+	bundleVersion        = "10"
+	distributionEndpoint = "distribution/"
+)
 
 var (
 	distributionDetails *config.ServerDetails
 	distAuth            auth.ServiceDetails
 	distHttpDetails     httputils.HttpClientDetails
 	// JFrog CLI for Distribution commands
-	distributionCli *tests.JfrogCli
+	distributionCli *coreTests.JfrogCli
 )
 
 func InitDistributionTests() {
@@ -42,30 +51,24 @@ func CleanDistributionTests() {
 }
 
 func authenticateDistribution() string {
+	*tests.JfrogUrl = clientUtils.AddTrailingSlashIfNeeded(*tests.JfrogUrl)
+	distributionDetails = &config.ServerDetails{DistributionUrl: *tests.JfrogUrl + distributionEndpoint}
+	cred := "--url=" + distributionDetails.DistributionUrl
+	if *tests.JfrogAccessToken != "" {
+		distributionDetails.AccessToken = *tests.JfrogAccessToken
+		cred += " --access-token=" + *tests.JfrogAccessToken
+	} else {
+		distributionDetails.Password = *tests.JfrogPassword
+		cred += " --password=" + *tests.JfrogPassword
+	}
 	// Due to a bug in distribution when authenticate with a multi-scope token,
 	// we must send a username as well as token or password.
-	distributionDetails = &config.ServerDetails{
-		DistributionUrl: *tests.RtDistributionUrl,
-		User:            *tests.RtUser,
-	}
-	cred := "--dist-url=" + *tests.RtDistributionUrl + " --user=" + *tests.RtUser
-
-	// Prefer the distribution token if provided.
-	distributionDetails.AccessToken = *tests.RtDistributionAccessToken
-	if distributionDetails.AccessToken == "" {
-		distributionDetails.AccessToken = *tests.RtAccessToken
-	}
-
-	if distributionDetails.AccessToken != "" {
-		cred += " --access-token=" + distributionDetails.AccessToken
-	} else {
-		distributionDetails.Password = *tests.RtPassword
-		cred += " --password=" + *tests.RtPassword
-	}
+	distributionDetails.User = *tests.JfrogUser
+	cred += " --user=" + *tests.JfrogUser
 
 	var err error
 	if distAuth, err = distributionDetails.CreateDistAuthConfig(); err != nil {
-		coreutils.ExitOnErr(errors.New("Failed while attempting to authenticate with Artifactory: " + err.Error()))
+		coreutils.ExitOnErr(errors.New("Failed while attempting to authenticate with Distribution: " + err.Error()))
 	}
 	distributionDetails.DistributionUrl = distAuth.GetUrl()
 	distHttpDetails = distAuth.CreateHttpClientDetails()
@@ -76,9 +79,8 @@ func initDistributionCli() {
 	if distributionCli != nil {
 		return
 	}
-	*tests.RtDistributionUrl = utils.AddTrailingSlashIfNeeded(*tests.RtDistributionUrl)
 	cred := authenticateDistribution()
-	distributionCli = tests.NewJfrogCli(execMain, "jfrog rt", cred)
+	distributionCli = coreTests.NewJfrogCli(execMain, "jfrog ds", cred)
 }
 
 func initDistributionTest(t *testing.T) {
@@ -88,8 +90,12 @@ func initDistributionTest(t *testing.T) {
 }
 
 func cleanDistributionTest(t *testing.T) {
-	distributionCli.Exec("rbdel", tests.BundleName, bundleVersion, "--site=*", "--delete-from-dist", "--quiet")
-	inttestutils.WaitForDeletion(t, tests.BundleName, bundleVersion, distHttpDetails)
+	err := distributionCli.Exec("rbdel", tests.BundleName, bundleVersion, "--site=*", "--delete-from-dist", "--quiet", "--sync")
+	if err != nil {
+		// If release bundle already deleted during the tests no need to fail here
+		assert.ErrorContains(t, err, fmt.Sprintf("Release Bundle Version '%s/%s' was not found", tests.BundleName, bundleVersion))
+	}
+
 	inttestutils.CleanDistributionRepositories(t, serverDetails)
 	tests.CleanFileSystem()
 }
@@ -103,8 +109,8 @@ func TestBundleAsyncDistDownload(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create and distribute release bundle
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--create-repo")
 	inttestutils.WaitForDistribution(t, tests.BundleName, bundleVersion, distHttpDetails)
 
 	// Download by bundle version, b2 and b3 should not be downloaded, b1 should
@@ -128,13 +134,12 @@ func TestBundleDownloadUsingSpec(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create release bundle
-	distributionRules, err := tests.CreateSpec(tests.DistributionRules)
-	assert.NoError(t, err)
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--dist-rules="+distributionRules, "--sync")
+	distributionRulesPath := filepath.Join(tests.GetTestResourcesPath(), "distribution", tests.DistributionRules)
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--dist-rules="+distributionRulesPath, "--sync", "--create-repo")
 
-	// Download by bundle version, b2 and b3 should not be downloaded, b1 should
-	specFile, err = tests.CreateSpec(tests.BundleDownloadSpec)
+	// Download by bundle version with gpg validation, b2 and b3 should not be downloaded, b1 should
+	specFile, err = tests.CreateSpec(tests.BundleDownloadGpgSpec)
 	assert.NoError(t, err)
 	runRt(t, "dl", "--spec="+specFile)
 
@@ -158,8 +163,8 @@ func TestBundleCreateByAql(t *testing.T) {
 	// Create release bundle by AQL
 	spec, err := tests.CreateSpec(tests.DistributionCreateByAql)
 	assert.NoError(t, err)
-	runRb(t, "rbc", tests.BundleName, bundleVersion, "--spec="+spec, "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, "--spec="+spec, "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Download by bundle version, b2 and b3 should not be downloaded, b1 should
 	specFile, err = tests.CreateSpec(tests.BundleDownloadSpec)
@@ -184,8 +189,8 @@ func TestBundleDownloadNoPattern(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create release bundle
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Download by bundle name and version with pattern "*", b2 and b3 should not be downloaded, b1 should
 	runRt(t, "dl", "*", "out/download/simple_by_build/data/", "--bundle="+tests.BundleName+"/"+bundleVersion, "--flat")
@@ -195,9 +200,10 @@ func TestBundleDownloadNoPattern(t *testing.T) {
 	err = tests.ValidateListsIdentical(tests.GetBuildSimpleDownload(), paths)
 	assert.NoError(t, err)
 
-	// Download by bundle name and version version without pattern, b2 and b3 should not be downloaded, b1 should
+	// Download by bundle name and version without pattern, b2 and b3 should not be downloaded, b1 should
 	tests.CleanFileSystem()
 	specFile, err = tests.CreateSpec(tests.BundleDownloadSpecNoPattern)
+	assert.NoError(t, err)
 	runRt(t, "dl", "--spec="+specFile, "--flat")
 
 	// Validate files are downloaded by bundle version
@@ -218,8 +224,8 @@ func TestBundleExclusions(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create release bundle. Include b1.in and b2.in. Exclude b3.in.
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b*.in", "--sign", "--exclusions=*b3.in")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b*.in", "--sign", "--exclusions=*b3.in")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Download by bundle version, b2 and b3 should not be downloaded, b1 should
 	runRt(t, "dl", tests.DistRepo1+"/data/*", tests.Out+fileutils.GetFileSeparator()+"download"+fileutils.GetFileSeparator()+"simple_by_build"+fileutils.GetFileSeparator(), "--bundle="+tests.BundleName+"/"+bundleVersion, "--exclusions=*b2.in")
@@ -245,8 +251,8 @@ func TestBundleCopy(t *testing.T) {
 	runRt(t, "u", "--spec="+specFileB)
 
 	// Create release bundle
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/a*", "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/a*", "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Copy by bundle name and version
 	specFile, err := tests.CreateSpec(tests.CopyByBundleSpec)
@@ -256,7 +262,7 @@ func TestBundleCopy(t *testing.T) {
 	// Validate files are copied by bundle version
 	spec, err := tests.CreateSpec(tests.CopyByBundleAssertSpec)
 	assert.NoError(t, err)
-	verifyExistInArtifactory(tests.GetBundleCopyExpected(), spec, t)
+	inttestutils.VerifyExistInArtifactory(tests.GetBundleCopyExpected(), spec, serverDetails, t)
 
 	// Cleanup
 	cleanDistributionTest(t)
@@ -269,8 +275,8 @@ func TestBundleSetProperties(t *testing.T) {
 	runRt(t, "u", "testdata/a/a1.in", tests.DistRepo1+"/a.in")
 
 	// Create release bundle
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/a.in", "--sign")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/a.in", "--sign")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Set the 'prop=red' property to the file.
 	runRt(t, "sp", tests.DistRepo1+"/a.*", "prop=red", "--bundle="+tests.BundleName+"/"+bundleVersion)
@@ -304,12 +310,12 @@ func TestSignReleaseBundle(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create a release bundle without --sign and make sure it is not signed
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in")
 	distributableResponse := inttestutils.GetLocalBundle(t, tests.BundleName, bundleVersion, distHttpDetails)
 	inttestutils.AssertReleaseBundleOpen(t, distributableResponse)
 
 	// Sign the release bundle and make sure it is signed
-	runRb(t, "rbs", tests.BundleName, bundleVersion)
+	runDs(t, "rbs", tests.BundleName, bundleVersion)
 	distributableResponse = inttestutils.GetLocalBundle(t, tests.BundleName, bundleVersion, distHttpDetails)
 	inttestutils.AssertReleaseBundleSigned(t, distributableResponse)
 
@@ -326,11 +332,11 @@ func TestBundleDeleteLocal(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create a release bundle
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
 
 	// Delete release bundle locally
-	runRb(t, "rbdel", tests.BundleName, bundleVersion, "--site=*", "--delete-from-dist", "--quiet")
+	runDs(t, "rbdel", tests.BundleName, bundleVersion, "--site=*", "--delete-from-dist", "--quiet")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, false, distHttpDetails)
 
 	// Cleanup
@@ -346,17 +352,24 @@ func TestUpdateReleaseBundle(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create a release bundle with b2.in
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
 
 	// Update release bundle to have b1.in
-	runRb(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
+	runDs(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign")
 
 	// Distribute release bundle
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
-	// Download by bundle version, b2 and b3 should not be downloaded, b1 should
-	runRt(t, "dl", tests.DistRepo1+"/data/*", tests.Out+fileutils.GetFileSeparator()+"download"+fileutils.GetFileSeparator()+"simple_by_build"+fileutils.GetFileSeparator(), "--bundle="+tests.BundleName+"/"+bundleVersion)
+	// GPG validation for release bundle
+	keyPath := filepath.Join(tests.GetTestResourcesPath(), "distribution", "public.key.1")
+	wrongKeyPath := filepath.Join(tests.GetTestResourcesPath(), "distribution", "public.key.2")
+	// Flag --gpg-key with no --bundle flag - returns error
+	runRtCmdExpectError(t, "dl", tests.DistRepo1+"/data/*", tests.Out+fileutils.GetFileSeparator()+"download"+fileutils.GetFileSeparator()+"simple_by_build"+fileutils.GetFileSeparator(), "--gpg-key="+wrongKeyPath)
+	// Validate with the wrong key - returns error
+	runRtCmdExpectError(t, "dl", tests.DistRepo1+"/data/*", tests.Out+fileutils.GetFileSeparator()+"download"+fileutils.GetFileSeparator()+"simple_by_build"+fileutils.GetFileSeparator(), "--bundle="+tests.BundleName+"/"+bundleVersion, "--gpg-key="+wrongKeyPath)
+	// Download by bundle version with the correct key, b2 and b3 should not be downloaded, b1 should
+	runRt(t, "dl", tests.DistRepo1+"/data/*", tests.Out+fileutils.GetFileSeparator()+"download"+fileutils.GetFileSeparator()+"simple_by_build"+fileutils.GetFileSeparator(), "--bundle="+tests.BundleName+"/"+bundleVersion, "--gpg-key="+keyPath)
 
 	// Validate files are downloaded by bundle version
 	paths, _ := fileutils.ListFilesRecursiveWalkIntoDirSymlink(tests.Out, false)
@@ -378,7 +391,7 @@ func TestCreateBundleText(t *testing.T) {
 	// Create a release bundle with release notes and description
 	releaseNotesPath := filepath.Join(tests.GetTestResourcesPath(), "distribution", "releasenotes.md")
 	description := "thisIsADescription"
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--release-notes-path="+releaseNotesPath, "--desc="+description)
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--release-notes-path="+releaseNotesPath, "--desc="+description)
 
 	// Validate release notes and description
 	distributableResponse := inttestutils.GetLocalBundle(t, tests.BundleName, bundleVersion, distHttpDetails)
@@ -402,9 +415,9 @@ func TestCreateBundleProps(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create and distribute release bundle with added props
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--target-props=key1=val1;key2=val2,val3", "--sign")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--target-props=key1=val1;key2=val2,val3", "--sign")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Verify props are added to the distributes artifact
 	verifyExistInArtifactoryByProps(tests.GetBundlePropsExpected(), tests.DistRepo1+"/data/", "key1=val1;key2=val2;key2=val3", t)
@@ -421,10 +434,10 @@ func TestUpdateBundleProps(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create, update and distribute release bundle with added props
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*")
-	runRb(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--target-props=key1=val1", "--sign")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*")
+	runDs(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/*", "--target-props=key1=val1", "--sign")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync", "--create-repo")
 
 	// Verify props are added to the distributes artifact
 	verifyExistInArtifactoryByProps(tests.GetBundlePropsExpected(), tests.DistRepo1+"/data/", "key1=val1", t)
@@ -432,7 +445,7 @@ func TestUpdateBundleProps(t *testing.T) {
 	cleanDistributionTest(t)
 }
 
-func TestBundlePathMapping(t *testing.T) {
+func TestBundlePathMappingFromPatternAndTarget(t *testing.T) {
 	initDistributionTest(t)
 
 	// Upload files
@@ -441,13 +454,35 @@ func TestBundlePathMapping(t *testing.T) {
 	runRt(t, "u", "--spec="+specFile)
 
 	// Create and distribute release bundle with path mapping from <DistRepo1>/data/ to <DistRepo2>/target/
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/(*)", "--sign", "--target="+tests.DistRepo2+"/target/{1}")
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/(*)", "--sign", "--target="+tests.DistRepo2+"/target/{1}")
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
 
 	// Validate files are distributed to the target mapping
 	spec, err := tests.CreateSpec(tests.DistributionMappingDownload)
 	assert.NoError(t, err)
-	verifyExistInArtifactory(tests.GetBundleMappingExpected(), spec, t)
+	inttestutils.VerifyExistInArtifactory(tests.GetBundleMappingExpected(), spec, serverDetails, t)
+
+	cleanDistributionTest(t)
+}
+
+func TestBundlePathMappingFromPatternAndTargetUsingSpec(t *testing.T) {
+	initDistributionTest(t)
+
+	// Upload files
+	specFile, err := tests.CreateSpec(tests.DistributionUploadSpecB)
+	assert.NoError(t, err)
+	runRt(t, "u", "--spec="+specFile)
+
+	// Create and distribute release bundle with path mapping from <DistRepo1>/data/ to <DistRepo2>/target/
+	spec, err := tests.CreateSpec(tests.DistributionCreateWithPatternAndTarget)
+	assert.NoError(t, err)
+	runDs(t, "rbc", tests.BundleName, bundleVersion, "--sign", "--spec="+spec)
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+
+	// Validate files are distributed to the target mapping
+	spec, err = tests.CreateSpec(tests.DistributionMappingDownload)
+	assert.NoError(t, err)
+	inttestutils.VerifyExistInArtifactory(tests.GetBundleMappingExpected(), spec, serverDetails, t)
 
 	cleanDistributionTest(t)
 }
@@ -463,13 +498,13 @@ func TestBundlePathMappingUsingSpec(t *testing.T) {
 	// Create and distribute release bundle with path mapping from <DistRepo1>/data/ to <DistRepo2>/target/
 	spec, err := tests.CreateSpec(tests.DistributionCreateWithMapping)
 	assert.NoError(t, err)
-	runRb(t, "rbc", tests.BundleName, bundleVersion, "--sign", "--spec="+spec)
-	runRb(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, "--sign", "--spec="+spec)
+	runDs(t, "rbd", tests.BundleName, bundleVersion, "--site=*", "--sync")
 
 	// Validate files are distributed to the target mapping
 	spec, err = tests.CreateSpec(tests.DistributionMappingDownload)
 	assert.NoError(t, err)
-	verifyExistInArtifactory(tests.GetBundleMappingExpected(), spec, t)
+	inttestutils.VerifyExistInArtifactory(tests.GetBundleMappingExpected(), spec, serverDetails, t)
 
 	cleanDistributionTest(t)
 }
@@ -482,12 +517,12 @@ func TestReleaseBundleCreateDetailedSummary(t *testing.T) {
 	assert.NoError(t, err)
 	runRt(t, "u", "--spec="+specFile)
 
-	buffer, previousLog := tests.RedirectLogOutputToBuffer()
+	buffer, _, previousLog := coreTests.RedirectLogOutputToBuffer()
 	// Restore previous logger when the function returns
 	defer log.SetLogger(previousLog)
 
 	// Create a release bundle with b2.in
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in", "--sign", "--detailed-summary")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in", "--sign", "--detailed-summary")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
 
 	tests.VerifySha256DetailedSummaryFromBuffer(t, buffer, previousLog)
@@ -504,16 +539,16 @@ func TestReleaseBundleUpdateDetailedSummary(t *testing.T) {
 	assert.NoError(t, err)
 	runRt(t, "u", "--spec="+specFile)
 
-	buffer, previousLog := tests.RedirectLogOutputToBuffer()
+	buffer, _, previousLog := coreTests.RedirectLogOutputToBuffer()
 	// Restore previous logger when the function returns
 	defer log.SetLogger(previousLog)
 
 	// Create a release bundle with b2.in
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
 
 	// Update release bundle to have b1.in
-	runRb(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign", "--detailed-summary")
+	runDs(t, "rbu", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b1.in", "--sign", "--detailed-summary")
 
 	tests.VerifySha256DetailedSummaryFromBuffer(t, buffer, previousLog)
 
@@ -529,16 +564,16 @@ func TestReleaseBundleSignDetailedSummary(t *testing.T) {
 	assert.NoError(t, err)
 	runRt(t, "u", "--spec="+specFile)
 
-	buffer, previousLog := tests.RedirectLogOutputToBuffer()
+	buffer, _, previousLog := coreTests.RedirectLogOutputToBuffer()
 	// Restore previous logger when the function returns
 	defer log.SetLogger(previousLog)
 
 	// Create a release bundle with b2.in
-	runRb(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
+	runDs(t, "rbc", tests.BundleName, bundleVersion, tests.DistRepo1+"/data/b2.in")
 	inttestutils.VerifyLocalBundleExistence(t, tests.BundleName, bundleVersion, true, distHttpDetails)
 
 	// Update release bundle to have b1.in
-	runRb(t, "rbs", tests.BundleName, bundleVersion, "--detailed-summary")
+	runDs(t, "rbs", tests.BundleName, bundleVersion, "--detailed-summary")
 
 	tests.VerifySha256DetailedSummaryFromBuffer(t, buffer, previousLog)
 
@@ -546,14 +581,55 @@ func TestReleaseBundleSignDetailedSummary(t *testing.T) {
 	cleanDistributionTest(t)
 }
 
-// Run `jfrog rt rb*` command`. The first arg is the distribution command, such as 'rbc', 'rbu', etc.
-func runRb(t *testing.T, args ...string) {
+func TestDistributeSyncTimeout(t *testing.T) {
+	initDistributionTest(t)
+
+	trackerId := "123"
+	statusRequestsReceived := 0
+	testServer, mockServerDetails, _ := coreTestUtils.CreateDsRestsMockServer(t, func(w http.ResponseWriter, r *http.Request) {
+		if r.RequestURI == "/api/v1/distribution/"+tests.BundleName+"/"+bundleVersion {
+			w.WriteHeader(http.StatusOK)
+			content, err := json.Marshal(distribution.DistributionResponseBody{TrackerId: json.Number(trackerId)})
+			assert.NoError(t, err)
+			_, err = w.Write(content)
+			assert.NoError(t, err)
+			return
+		}
+		if r.RequestURI == "/api/v1/release_bundle/"+tests.BundleName+"/"+bundleVersion+"/distribution/"+trackerId {
+			statusRequestsReceived++
+			w.WriteHeader(http.StatusOK)
+			content, err := json.Marshal(distribution.DistributionStatusResponse{Status: distribution.InProgress})
+			assert.NoError(t, err)
+			_, err = w.Write(content)
+			assert.NoError(t, err)
+			return
+		}
+	})
+	defer testServer.Close()
+
+	maxWaitMinutes := 1
+	distributionRulesPath := filepath.Join(tests.GetTestResourcesPath(), "distribution", tests.DistributionRules)
+
+	mockDsCli := coreTests.NewJfrogCli(execMain, "jfrog ds", "--url="+mockServerDetails.DistributionUrl)
+	err := mockDsCli.Exec("rbd", tests.BundleName, bundleVersion, "--dist-rules="+distributionRulesPath, "--sync", "--max-wait-minutes="+strconv.Itoa(maxWaitMinutes), "--create-repo")
+	assert.ErrorContains(t, err, "executor timeout after")
+	assert.ErrorAs(t, err, &clientUtils.RetryExecutorTimeoutError{})
+
+	expectedStatusRequests := (maxWaitMinutes * 60 / distributionServices.DefaultDistributeSyncSleepIntervalSeconds) + 1
+	assert.Equal(t, expectedStatusRequests, statusRequestsReceived)
+
+	// Cleanup
+	cleanDistributionTest(t)
+}
+
+// Run `jfrog ds` command`. The first arg is the distribution command, such as 'rbc', 'rbu', etc.
+func runDs(t *testing.T, args ...string) {
 	err := distributionCli.Exec(args...)
 	assert.NoError(t, err)
 }
 
-// Run `jfrog rt` command
-func runRt(t *testing.T, args ...string) {
+// Run `jfrog rt` command and expected an error
+func runRtCmdExpectError(t *testing.T, args ...string) {
 	err := artifactoryCli.Exec(args...)
-	assert.NoError(t, err)
+	assert.Error(t, err)
 }
