@@ -1,10 +1,13 @@
 package buildaddgit
 
-const Description = "Collect VCS details from git and add them to a build."
+var Usage = []string{"rt bag [command options] <build name> <build number> [Path To .git]"}
 
-var Usage = []string{"jfrog rt bag [command options] <build name> <build number> [Path To .git]"}
+func GetDescription() string {
+	return "Collect VCS details from git and add them to a build."
+}
 
-const Arguments string = `	build name
+func GetArguments() string {
+	return `	build name
 		Build name.
 
 	build number
@@ -12,3 +15,4 @@ const Arguments string = `	build name
 
 	path to .git
 		Path to a directory containing the .git directory. If not specified, the .git directory is assumed to be in the current directory or in one of the parent directories.`
+}
