@@ -1,45 +1,42 @@
-package commands
+package cisetup
 
 import (
 	"bytes"
 	"encoding/json"
-	"errors"
 	"fmt"
+	"golang.org/x/exp/slices"
 	"os"
+	"path"
 	"path/filepath"
 	"strings"
 	"syscall"
 
-	"github.com/gookit/color"
-	"github.com/jfrog/jfrog-cli-core/artifactory/commands/permissiontarget"
-	"github.com/jfrog/jfrog-cli-core/artifactory/commands/usersmanagement"
-	"github.com/jfrog/jfrog-cli-core/general/cisetup"
-	"github.com/jfrog/jfrog-client-go/pipelines"
-	pipelinesservices "github.com/jfrog/jfrog-client-go/pipelines/services"
-	"github.com/jfrog/jfrog-client-go/utils"
-
 	"github.com/go-git/go-git/v5"
 	"github.com/go-git/go-git/v5/plumbing"
 	"github.com/go-git/go-git/v5/plumbing/transport"
 	"github.com/go-git/go-git/v5/plumbing/transport/http"
-	"github.com/jfrog/jfrog-cli-core/artifactory/commands/buildinfo"
-	rtutils "github.com/jfrog/jfrog-cli-core/artifactory/utils"
-	"github.com/jfrog/jfrog-cli-core/common/commands"
-	corecommoncommands "github.com/jfrog/jfrog-cli-core/common/commands"
-	"github.com/jfrog/jfrog-cli-core/utils/config"
-	utilsconfig "github.com/jfrog/jfrog-cli-core/utils/config"
-	"github.com/jfrog/jfrog-cli-core/utils/coreutils"
-	"github.com/jfrog/jfrog-cli-core/utils/ioutils"
+	"github.com/jfrog/jfrog-cli-core/v2/artifactory/commands/buildinfo"
+	"github.com/jfrog/jfrog-cli-core/v2/artifactory/commands/permissiontarget"
+	"github.com/jfrog/jfrog-cli-core/v2/artifactory/commands/usersmanagement"
+	"github.com/jfrog/jfrog-cli-core/v2/common/build"
+	coreCommonCommands "github.com/jfrog/jfrog-cli-core/v2/common/commands"
+	"github.com/jfrog/jfrog-cli-core/v2/general/cisetup"
+	repoutils "github.com/jfrog/jfrog-cli-core/v2/general/project"
+	utilsConfig "github.com/jfrog/jfrog-cli-core/v2/utils/config"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/coreutils"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/ioutils"
 	buildinfocmd "github.com/jfrog/jfrog-client-go/artifactory/buildinfo"
 	"github.com/jfrog/jfrog-client-go/artifactory/services"
 	clientConfig "github.com/jfrog/jfrog-client-go/config"
-	clientutils "github.com/jfrog/jfrog-client-go/utils"
+	"github.com/jfrog/jfrog-client-go/pipelines"
+	pipelinesservices "github.com/jfrog/jfrog-client-go/pipelines/services"
+	"github.com/jfrog/jfrog-client-go/utils"
 	"github.com/jfrog/jfrog-client-go/utils/errorutils"
 	"github.com/jfrog/jfrog-client-go/utils/io/fileutils"
 	"github.com/jfrog/jfrog-client-go/utils/log"
 	xrayservices "github.com/jfrog/jfrog-client-go/xray/services"
 	xrayutils "github.com/jfrog/jfrog-client-go/xray/services/utils"
-	"golang.org/x/crypto/ssh/terminal"
+	"golang.org/x/term"
 )
 
 const (
@@ -55,6 +52,7 @@ const (
 	ideUserPassPlaceholder   = "<INSERT-PASSWORD>"
 	ideUserEmailPlaceholder  = "<INSERT-EMAIL>"
 	createUserTemplate       = `jfrog rt user-create "%s" "%s" "%s" --users-groups="%s" --server-id="%s"`
+	maxRepoCreationAttempts  = 200
 )
 
 type CiSetupCommand struct {
@@ -82,13 +80,13 @@ func RunCiSetupCmd() error {
 func logBeginningInstructions() error {
 	instructions := []string{
 		"",
-		colorTitle("About this command"),
+		coreutils.PrintTitle("About this command"),
 		"This command sets up a basic CI pipeline which uses the JFrog Platform.",
 		"It currently supports Maven, Gradle and npm, but additional package managers will be added in the future.",
 		"The following CI providers are currently supported: JFrog Pipelines, Jenkins and GitHub Actions.",
 		"The command takes care of configuring JFrog Artifactory and JFrog Xray for you.",
 		"",
-		colorTitle("Important"),
+		coreutils.PrintTitle("Important"),
 		" 1. If you don't have a JFrog Platform instance with admin credentials, head over to https://jfrog.com/start-free/ and get one for free.",
 		" 2. When asked to provide credentials for your JFrog Platform and Git provider, please make sure the credentials have admin privileges.",
 		" 3. You can exit the command by hitting 'control + C' at any time. The values you provided before exiting are saved (with the exception of passwords and tokens) and will be set as defaults the next time you run the command.",
@@ -97,23 +95,17 @@ func logBeginningInstructions() error {
 	return writeToScreen(strings.Join(instructions, "\n"))
 }
 
-func inactivePipelinesNote() error {
+func inactivePipelinesNote(pipelinesUrl string) error {
 	instructions := []string{
 		"",
-		colorTitle("JFrog Pipelines"),
-		"It looks like your JFrog platform dose not include JFrog Pipelines or it is currently inactive.",
+		coreutils.PrintTitle("JFrog Pipelines"),
+		"It looks like your JFrog platform does not include JFrog Pipelines or it is currently inactive.",
+		pipelinesUrl,
 		"",
 	}
 	return writeToScreen(strings.Join(instructions, "\n"))
 }
 
-func colorTitle(title string) string {
-	if terminal.IsTerminal(int(os.Stderr.Fd())) {
-		return color.Green.Render(title)
-	}
-	return title
-}
-
 func (cc *CiSetupCommand) prepareConfigurationData() error {
 	// If data is nil, initialize a new one
 	if cc.data == nil {
@@ -128,11 +120,11 @@ func (cc *CiSetupCommand) prepareConfigurationData() error {
 
 func readVcsConf() (*cisetup.CiSetupData, error) {
 	conf := &cisetup.CiSetupData{}
-	path, err := coreutils.GetJfrogHomeDir()
+	homeDirPath, err := coreutils.GetJfrogHomeDir()
 	if err != nil {
 		return nil, err
 	}
-	confPath := filepath.Join(path, VcsConfigFile)
+	confPath := filepath.Join(homeDirPath, VcsConfigFile)
 	exists, err := fileutils.IsFileExists(confPath, false)
 	if err != nil {
 		return nil, err
@@ -149,7 +141,7 @@ func readVcsConf() (*cisetup.CiSetupData, error) {
 }
 
 func saveVcsConf(conf *cisetup.CiSetupData) error {
-	path, err := coreutils.GetJfrogHomeDir()
+	homeDirPath, err := coreutils.GetJfrogHomeDir()
 	if err != nil {
 		return err
 	}
@@ -162,7 +154,7 @@ func saveVcsConf(conf *cisetup.CiSetupData) error {
 	if err != nil {
 		return errorutils.CheckError(err)
 	}
-	err = os.WriteFile(filepath.Join(path, VcsConfigFile), []byte(content.String()), 0600)
+	err = os.WriteFile(filepath.Join(homeDirPath, VcsConfigFile), content.Bytes(), 0600)
 	return errorutils.CheckError(err)
 }
 
@@ -179,12 +171,12 @@ func (cc *CiSetupCommand) Run() error {
 		return err
 	}
 	// Ask the user which CI he tries to setup
-	err = cc.ciProviderPhase()
-	err = saveIfNoError(err, cc.data)
+	cc.ciProviderPhase()
+	err = saveVcsConf(cc.data)
 	if err != nil {
 		return err
 	}
-	// Interactively create Artifactory repository based on the detected technologies and on going user input
+	// Interactively create Artifactory repository based on the detected technologies and ongoing user input
 	err = cc.artifactoryConfigPhase()
 	err = saveIfNoError(err, cc.data)
 	if err != nil {
@@ -245,7 +237,7 @@ func saveIfNoError(errCheck error, conf *cisetup.CiSetupData) error {
 }
 
 func runIdePhase() error {
-	serverDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+	serverDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 	if err != nil {
 		return err
 	}
@@ -256,7 +248,7 @@ func runIdePhase() error {
 	return createPermissionTarget(serverDetails)
 }
 
-func createGroup(serverDetails *utilsconfig.ServerDetails) error {
+func createGroup(serverDetails *utilsConfig.ServerDetails) error {
 	log.Info("Creating group...")
 	groupCreateCmd := usersmanagement.NewGroupCreateCommand()
 	groupCreateCmd.SetName(ideGroupName).SetServerDetails(serverDetails).SetReplaceIfExists(false)
@@ -270,7 +262,7 @@ func createGroup(serverDetails *utilsconfig.ServerDetails) error {
 	return nil
 }
 
-func createPermissionTarget(serverDetails *utilsconfig.ServerDetails) error {
+func createPermissionTarget(serverDetails *utilsConfig.ServerDetails) error {
 	ptTemplate := fmt.Sprintf(permissionTargetTemplate, ideGroupName, permissionTargetName)
 	tempDir, err := fileutils.CreateTempDir()
 	if err != nil {
@@ -307,26 +299,27 @@ func getPipelinesToken() (string, error) {
 		if err != nil {
 			return "", err
 		}
-		byteToken, err = terminal.ReadPassword(int(syscall.Stdin))
+		//nolint:unconvert
+		byteToken, err = term.ReadPassword(int(syscall.Stdin))
 		if err != nil {
 			return "", errorutils.CheckError(err)
 		}
 		// New-line required after the access token input:
-		fmt.Println()
+		log.Output()
 	}
 	return string(byteToken), nil
 }
 
 func runConfigCmd() (err error) {
 	for {
-		configCmd := corecommoncommands.NewConfigCommand().SetInteractive(true).SetServerId(cisetup.ConfigServerId).SetEncPassword(true)
-		err = configCmd.Config()
+		configCmd := coreCommonCommands.NewConfigCommand(coreCommonCommands.AddOrEdit, cisetup.ConfigServerId).SetInteractive(true).SetEncPassword(true)
+		err = configCmd.Run()
 		if err != nil {
 			log.Error(err)
 			continue
 		}
 		// Validate JFrog credentials by execute get repo command
-		serviceDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+		serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 		if err != nil {
 			return err
 		}
@@ -339,10 +332,10 @@ func runConfigCmd() (err error) {
 }
 
 func (cc *CiSetupCommand) runJenkinsPhase() (string, error) {
-	generator := cisetup.JenkinsfileGenerator{
+	generator := cisetup.JenkinsfileDslGenerator{
 		SetupData: cc.data,
 	}
-	jenkinsfileBytes, jenkinsfileName, err := generator.Generate()
+	jenkinsfileBytes, jenkinsfileName, err := generator.GenerateDsl()
 	if err != nil {
 		return "", err
 	}
@@ -427,18 +420,18 @@ func (cc *CiSetupCommand) runPipelinesPhase() (string, error) {
 }
 
 func (cc *CiSetupCommand) saveCiConfigToFile(ciConfig []byte, fileName string) error {
-	path := filepath.Join(cc.data.LocalDirPath, fileName)
-	log.Info(fmt.Sprintf("Generating %s at: %q ...", fileName, path))
-	return os.WriteFile(path, ciConfig, 0644)
+	filePath := filepath.Join(cc.data.LocalDirPath, fileName)
+	log.Info(fmt.Sprintf("Generating %s at: %q ...", fileName, filePath))
+	return os.WriteFile(filePath, ciConfig, 0644)
 }
 
 func (cc *CiSetupCommand) getPipelinesCompletionInstruction(pipelinesFileName string) ([]string, error) {
-	serviceDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+	serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 	if err != nil {
 		return []string{}, err
 	}
 
-	return []string{"", colorTitle("Completing the setup"),
+	return []string{"", coreutils.PrintTitle("Completing the setup"),
 		"We configured the JFrog Platform and generated a pipelines.yml for you.",
 		"To complete the setup, add the new pipelines.yml to your git repository by running the following commands:",
 		"",
@@ -452,26 +445,22 @@ func (cc *CiSetupCommand) getPipelinesCompletionInstruction(pipelinesFileName st
 }
 
 func (cc *CiSetupCommand) getJenkinsCompletionInstruction() []string {
-	JenkinsCompletionInstruction := []string{"", colorTitle("Completing the setup"),
+	JenkinsCompletionInstruction := []string{"", coreutils.PrintTitle("Completing the setup"),
 		"We configured the JFrog Platform and generated a Jenkinsfile file for you under " + cc.data.LocalDirPath,
 		"To complete the setup, follow these steps:",
 		"* Open the Jenkinsfile for edit."}
-	// M2_HOME instructions relevant only for Maven
-	if cc.data.BuiltTechnology.Type == cisetup.Maven {
+	// HOME env instructions relevant only for Maven
+	if cc.data.BuiltTechnology.Type == coreutils.Maven || cc.data.BuiltTechnology.Type == coreutils.Gradle {
 		JenkinsCompletionInstruction = append(JenkinsCompletionInstruction,
-			"* Inside the 'environment' section, set the value of the M2_HOME variable,",
+			"* Inside the 'environment' section, set the value of the HOME ENV variable,",
 			"  to the Maven installation directory on the Jenkins agent (the directory which includes the 'bin' directory).")
 	}
 
 	JenkinsCompletionInstruction = append(JenkinsCompletionInstruction,
-		"* Inside the 'environment' section, set the value of the JFROG_CLI_BUILD_URL variable,",
-		"  so that it includes the URL to the job run console log in Jenkins.",
-		"  You may need to look at URLs of other job runs, to build the URL.",
 		"* If cloning the code from git requires credentials, modify the 'git' step as described",
 		"  in the comment inside the 'Clone' step.",
-		"* Define an environment variable named RT_USERNAME with your JFrog Platform username as its value.",
-		"* Create credentials with 'rt-password' as its ID, with your JFrog Platform password as",
-		"  its value. Read more about this here - https://www.jenkins.io/doc/book/using/using-credentials/",
+		"* Create credentials with 'rt-cred-id' as its ID in: Jenkins > Configure System > credentials > 'username with password' > ID: 'rt-cred-id' )",
+		"  Read more about this here - https://www.jenkins.io/doc/book/using/using-credentials/",
 		"* Add the new Jenkinsfile to your git repository by running the following commands:",
 		"",
 		"\t cd "+cc.data.LocalDirPath,
@@ -485,7 +474,7 @@ func (cc *CiSetupCommand) getJenkinsCompletionInstruction() []string {
 }
 
 func (cc *CiSetupCommand) getGithubActionsCompletionInstruction(githubActionFileName string) []string {
-	return []string{"", colorTitle("Completing the setup"),
+	return []string{"", coreutils.PrintTitle("Completing the setup"),
 		"We configured the JFrog Platform and generated a GitHub Actions workflow file",
 		"named " + cisetup.GithubActionsFileName + " for you under " + cisetup.GithubActionsDir + ".",
 		"",
@@ -507,11 +496,12 @@ func (cc *CiSetupCommand) getGithubActionsCompletionInstruction(githubActionFile
 }
 
 func (cc *CiSetupCommand) logCompletionInstruction(ciSpecificInstructions []string) error {
-	instructions := append(ciSpecificInstructions,
-		colorTitle("Allowing developers to access this pipeline from their IDE"),
+	instructions := append(slices.Clone(ciSpecificInstructions),
+		coreutils.PrintTitle("Allowing developers to access this pipeline from their IDE"),
 		"You have the option of viewing the new pipeline's runs from within IntelliJ IDEA.",
 		"To achieve this, follow these steps:",
-		" 1. Make sure the latest version of the JFrog Plugin is installed on IntelliJ IDEA.",
+		" 1. Make sure the latest version of the JFrog Plugin is installed on IntelliJ IDEA:",
+		"    https://www.jfrog.com/confluence/display/JFROG/JFrog+IntelliJ+IDEA+Plugin",
 		" 2. Create a JFrog user for the IDE by running the following command:",
 		"",
 		"\t "+fmt.Sprintf(createUserTemplate, ideUserName, ideUserPassPlaceholder, ideUserEmailPlaceholder, ideGroupName, cisetup.ConfigServerId),
@@ -529,31 +519,28 @@ func getPipelineUiPath(pipelinesUrl, pipelineName string) string {
 }
 
 func (cc *CiSetupCommand) publishFirstBuild() (err error) {
-	println("Everytime the new pipeline builds the code, it generates a build entity (also known as build-info) and stores it in Artifactory.")
-	ioutils.ScanFromConsole("Please choose a name for the build", &cc.data.BuildName, "${vcs.repo.name}-${branch}")
-	cc.data.BuildName = strings.Replace(cc.data.BuildName, "${vcs.repo.name}", cc.data.RepositoryName, -1)
-	cc.data.BuildName = strings.Replace(cc.data.BuildName, "${branch}", cc.data.GitBranch, -1)
+	cc.data.BuildName = fmt.Sprintf("%s-%s", cc.data.RepositoryName, cc.data.GitBranch)
 	// Run BAG Command (in order to publish the first, empty, build info)
-	buildAddGitConfigurationCmd := buildinfo.NewBuildAddGitCommand().SetDotGitPath(cc.data.LocalDirPath).SetServerId(cisetup.ConfigServerId) //.SetConfigFilePath(c.String("config"))
-	buildConfiguration := rtutils.BuildConfiguration{BuildName: cc.data.BuildName, BuildNumber: DefaultFirstBuildNumber}
-	buildAddGitConfigurationCmd = buildAddGitConfigurationCmd.SetBuildConfiguration(&buildConfiguration)
+	buildAddGitConfigurationCmd := buildinfo.NewBuildAddGitCommand().SetDotGitPath(cc.data.LocalDirPath).SetServerId(cisetup.ConfigServerId)
+	buildConfiguration := build.NewBuildConfiguration(cc.data.BuildName, DefaultFirstBuildNumber, "", "")
+	buildAddGitConfigurationCmd = buildAddGitConfigurationCmd.SetBuildConfiguration(buildConfiguration)
 	log.Info("Generating an initial build-info...")
-	err = commands.Exec(buildAddGitConfigurationCmd)
+	err = coreCommonCommands.Exec(buildAddGitConfigurationCmd)
 	if err != nil {
 		return err
 	}
 	// Run BP Command.
-	serviceDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+	serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 	if err != nil {
 		return err
 	}
 	buildInfoConfiguration := buildinfocmd.Configuration{DryRun: false}
-	buildPublishCmd := buildinfo.NewBuildPublishCommand().SetServerDetails(serviceDetails).SetBuildConfiguration(&buildConfiguration).SetConfig(&buildInfoConfiguration)
-	return commands.Exec(buildPublishCmd)
+	buildPublishCmd := buildinfo.NewBuildPublishCommand().SetServerDetails(serviceDetails).SetBuildConfiguration(buildConfiguration).SetConfig(&buildInfoConfiguration)
+	return coreCommonCommands.Exec(buildPublishCmd)
 }
 
 func (cc *CiSetupCommand) xrayConfigPhase() (err error) {
-	serviceDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+	serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 	if err != nil {
 		return err
 	}
@@ -564,6 +551,9 @@ func (cc *CiSetupCommand) xrayConfigPhase() (err error) {
 	// Index the build.
 	buildsToIndex := []string{cc.data.BuildName}
 	err = xrayManager.AddBuildsToIndexing(buildsToIndex)
+	if err != nil {
+		return err
+	}
 	// Create new default policy.
 	policyParams := xrayutils.NewPolicyParams()
 	policyParams.Name = "ci-pipeline-security-policy"
@@ -579,11 +569,10 @@ func (cc *CiSetupCommand) xrayConfigPhase() (err error) {
 	err = xrayManager.CreatePolicy(policyParams)
 	if err != nil {
 		// In case the error is from type PolicyAlreadyExistsError, we should continue with the regular flow.
-		if _, ok := err.(*xrayservices.PolicyAlreadyExistsError); !ok {
+		if paeErr, ok := err.(*xrayservices.PolicyAlreadyExistsError); !ok {
 			return err
 		} else {
-			log.Debug(err.(*xrayservices.PolicyAlreadyExistsError).InnerError)
-			err = nil
+			log.Debug(paeErr.InnerError)
 		}
 	}
 	// Create new default watcher.
@@ -602,10 +591,10 @@ func (cc *CiSetupCommand) xrayConfigPhase() (err error) {
 	err = xrayManager.CreateWatch(watchParams)
 	if err != nil {
 		// In case the error is from type WatchAlreadyExistsError, we should continue with the regular flow.
-		if _, ok := err.(*xrayservices.WatchAlreadyExistsError); !ok {
+		if waeErr, ok := err.(*xrayservices.WatchAlreadyExistsError); !ok {
 			return err
 		} else {
-			log.Debug(err.(*xrayservices.WatchAlreadyExistsError).InnerError)
+			log.Debug(waeErr.InnerError)
 			err = nil
 		}
 	}
@@ -618,8 +607,8 @@ func (cc *CiSetupCommand) artifactoryConfigPhase() (err error) {
 		return err
 	}
 	// First create repositories for the selected technology.
-	for tech, detected := range cc.data.DetectedTechnologies {
-		if detected && coreutils.AskYesNo(fmt.Sprintf("Would you like to use %s to build the code?", tech), true) {
+	for tech := range cc.data.DetectedTechnologies {
+		if coreutils.AskYesNo(fmt.Sprintf("Would you like to use %s to build the code?", tech), true) {
 			cc.data.BuiltTechnology = &cisetup.TechnologyInfo{Type: tech}
 			err = cc.interactivelyCreateRepos(tech)
 			if err != nil {
@@ -629,28 +618,18 @@ func (cc *CiSetupCommand) artifactoryConfigPhase() (err error) {
 			return nil
 		}
 	}
-	return errorutils.CheckError(errors.New("at least one of the supported technologies is expected to be chosen for building"))
+	return errorutils.CheckErrorf("at least one of the supported technologies is expected to be chosen for building")
 }
 
 func (cc *CiSetupCommand) printDetectedTechs() error {
 	var techs []string
-	for tech, detected := range cc.data.DetectedTechnologies {
-		if detected {
-			techs = append(techs, string(tech))
-		}
+	for tech := range cc.data.DetectedTechnologies {
+		techs = append(techs, string(tech))
 	}
 	if len(techs) == 0 {
-		return errorutils.CheckError(errors.New("no supported technology was found in the project"))
+		return errorutils.CheckErrorf("no supported technology was found in the project")
 	}
-	return writeToScreen("The next step is to provide the commands to build your code. It looks like the code is built with " + getExplicitTechsListByNumber(techs) + ".\n")
-}
-
-// Get the explicit list of technologies, for ex: "maven, gradle and npm"
-func getExplicitTechsListByNumber(techs []string) string {
-	if len(techs) == 1 {
-		return techs[0]
-	}
-	return strings.Join(techs[0:len(techs)-1], ", ") + " and " + techs[len(techs)-1]
+	return writeToScreen("The next step is to provide the commands to build your code. It looks like the code is built with " + coreutils.ListToText(techs) + ".\n")
 }
 
 func (cc *CiSetupCommand) getBuildCmd() {
@@ -666,33 +645,81 @@ func (cc *CiSetupCommand) getBuildCmd() {
 	ioutils.ScanFromConsole(prompt, &cc.data.BuiltTechnology.BuildCmd, defaultBuildCmd)
 }
 
-func (cc *CiSetupCommand) interactivelyCreateRepos(technologyType cisetup.Technology) (err error) {
-	serviceDetails, err := utilsconfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
-	if err != nil {
-		return err
-	}
-	// Get all relevant remotes to choose from
-	remoteRepos, err := GetAllRepos(serviceDetails, Remote, string(technologyType))
-	if err != nil {
-		return err
-	}
-	shouldPromptSelection := len(*remoteRepos) > 0
-	var remoteRepo string
+func getRepoSelectionFromUser(repos *[]services.RepositoryDetails, promptString string) (repo string, err error) {
+	shouldPromptSelection := len(*repos) > 0
 	if shouldPromptSelection {
-		// Ask if the user would like us to create a new remote or to choose from the exist repositories list
-		remoteRepo, err = promptARepoSelection(remoteRepos, "Select remote repository")
+		// Ask if the user would like us to create a new repo or to choose from the existing repositories list
+		repo, err = promptARepoSelection(repos, promptString)
+		if err != nil {
+			return "", err
+		}
+	} else {
+		repo = NewRepository
+	}
+	return repo, nil
+}
+
+func handleNewLocalRepository(serviceDetails *utilsConfig.ServerDetails, technologyType coreutils.Technology) (repo string) {
+	// Create local repository
+	for {
+		var newLocalRepo string
+		ioutils.ScanFromConsole("Repository Name", &newLocalRepo, repoutils.RepoDefaultName[technologyType][repoutils.Local])
+		err := CreateLocalRepo(serviceDetails, technologyType, newLocalRepo)
+		if err != nil {
+			log.Error(err)
+		} else {
+			return newLocalRepo
+		}
+	}
+}
+
+func (cc *CiSetupCommand) interactivelyCreateRepos(technologyType coreutils.Technology) (err error) {
+	serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+	if err != nil {
+		return err
+	}
+	// Get all relevant local to choose from
+	localRepos, err := GetAllRepos(serviceDetails, repoutils.Local, string(technologyType))
+	if err != nil {
+		return err
+	}
+	deployerRepoType := ""
+	if technologyType == coreutils.Maven {
+		deployerRepoType = "releases "
+	}
+	localRepo, err := getRepoSelectionFromUser(localRepos, fmt.Sprintf("Create or select an Artifactory %sRepository to deploy the build artifacts to", deployerRepoType))
+	if err != nil {
+		return err
+	}
+	if localRepo == NewRepository {
+		localRepo = handleNewLocalRepository(serviceDetails, technologyType)
+	}
+	cc.data.BuiltTechnology.LocalReleasesRepo = localRepo
+	if technologyType == coreutils.Maven {
+		localRepo, err = getRepoSelectionFromUser(localRepos, "Create or select an Artifactory snapshots Repository to deploy the build artifacts to")
 		if err != nil {
 			return err
 		}
-	} else {
-		remoteRepo = NewRepository
+		if localRepo == NewRepository {
+			localRepo = handleNewLocalRepository(serviceDetails, technologyType)
+		}
+	}
+	cc.data.BuiltTechnology.LocalSnapshotsRepo = localRepo
+	// Get all relevant remotes to choose from
+	remoteRepos, err := GetAllRepos(serviceDetails, repoutils.Remote, string(technologyType))
+	if err != nil {
+		return err
+	}
+	remoteRepo, err := getRepoSelectionFromUser(remoteRepos, "Create or select an Artifactory remote repository to resolve (download) 3rd party dependencies for your build")
+	if err != nil {
+		return err
 	}
 	// The user choose to create a new remote repo
 	if remoteRepo == NewRepository {
 		for {
 			var repoName, repoUrl string
-			ioutils.ScanFromConsole("Repository Name", &repoName, GetRemoteDefaultName(technologyType))
-			ioutils.ScanFromConsole("Repository URL", &repoUrl, GetRemoteDefaultUrl(technologyType))
+			ioutils.ScanFromConsole("Repository Name", &repoName, repoutils.RepoDefaultName[technologyType][repoutils.Remote])
+			ioutils.ScanFromConsole("Repository URL", &repoUrl, repoutils.RepoDefaultName[technologyType][repoutils.RemoteUrl])
 			err = CreateRemoteRepo(serviceDetails, technologyType, repoName, repoUrl)
 			if err != nil {
 				log.Error(err)
@@ -701,7 +728,7 @@ func (cc *CiSetupCommand) interactivelyCreateRepos(technologyType cisetup.Techno
 				for {
 					// Create a new virtual repository as well
 					ioutils.ScanFromConsole(fmt.Sprintf("Choose a name for a new virtual repository which will include %q remote repo", remoteRepo),
-						&repoName, GetVirtualDefaultName(technologyType))
+						&repoName, repoutils.RepoDefaultName[technologyType][repoutils.Virtual])
 					err = CreateVirtualRepo(serviceDetails, technologyType, repoName, remoteRepo)
 					if err != nil {
 						log.Error(err)
@@ -715,47 +742,40 @@ func (cc *CiSetupCommand) interactivelyCreateRepos(technologyType cisetup.Techno
 		}
 	}
 	// Else, the user choose an existing remote repo
-	virtualRepos, err := GetAllRepos(serviceDetails, Virtual, string(technologyType))
+	virtualRepos, err := GetAllRepos(serviceDetails, repoutils.Virtual, string(technologyType))
+	chosenVirtualRepo := ""
 	if err != nil {
 		return err
 	}
-	shouldPromptSelection = len(*virtualRepos) > 0
-	var virtualRepo string
-	if shouldPromptSelection {
-		// Ask if the user would like us to create a new virtual or to choose from the exist repositories list
-		virtualRepo, err = promptARepoSelection(virtualRepos, fmt.Sprintf("Select a virtual repository, which includes %s or choose to create a new repo:", remoteRepo))
+	for _, repo := range *virtualRepos {
+		virtualRepo, err := GetVirtualRepo(serviceDetails, repo.Key)
 		if err != nil {
 			return err
 		}
-	} else {
-		virtualRepo = NewRepository
+		if contains(virtualRepo.Repositories, remoteRepo) {
+			chosenVirtualRepo = repo.Key
+		}
 	}
-	if virtualRepo == NewRepository {
-		// Create virtual repository
-		for {
-			var repoName string
-			ioutils.ScanFromConsole("Repository Name", &repoName, GetVirtualDefaultName(technologyType))
-			err = CreateVirtualRepo(serviceDetails, technologyType, repoName, remoteRepo)
-			if err != nil {
-				log.Error(err)
-			} else {
-				virtualRepo = repoName
+	if chosenVirtualRepo == "" {
+		virtualRepoName := repoutils.RepoDefaultName[technologyType][repoutils.Virtual]
+		for i := 1; i < maxRepoCreationAttempts; i++ {
+			_, err := GetVirtualRepo(serviceDetails, virtualRepoName)
+			if err == nil {
+				err = CreateVirtualRepo(serviceDetails, technologyType, virtualRepoName, remoteRepo)
+				if err != nil {
+					return err
+				}
 				break
+			} else {
+				virtualRepoName = fmt.Sprintf("%s-%d", virtualRepoName, i)
 			}
 		}
-	} else {
-		// Validate that the chosen virtual repo contains the chosen remote repo
-		chosenVirtualRepo, err := GetVirtualRepo(serviceDetails, virtualRepo)
 		if err != nil {
 			return err
 		}
-		if !contains(chosenVirtualRepo.Repositories, remoteRepo) {
-			log.Error(fmt.Sprintf("The chosen virtual repo %q does not include the chosen remote repo %q", virtualRepo, remoteRepo))
-			return cc.interactivelyCreateRepos(technologyType)
-		}
+		chosenVirtualRepo = virtualRepoName
 	}
-	// Saves the new created repo name (key) in the results data structure.
-	cc.data.BuiltTechnology.VirtualRepo = virtualRepo
+	cc.data.BuiltTechnology.VirtualRepo = chosenVirtualRepo
 	return
 }
 
@@ -765,8 +785,8 @@ func promptARepoSelection(repoDetails *[]services.RepositoryDetails, promptMsg s
 	for _, repo := range *repoDetails {
 		selectableItems = append(selectableItems, ioutils.PromptItem{Option: repo.Key, TargetValue: &selectedRepoName, DefaultValue: repo.Url})
 	}
-	println(promptMsg)
-	err = ioutils.SelectString(selectableItems, "", func(item ioutils.PromptItem) {
+	log.Output(promptMsg)
+	err = ioutils.SelectString(selectableItems, "", true, func(item ioutils.PromptItem) {
 		*item.TargetValue = item.Option
 	})
 	return
@@ -785,8 +805,8 @@ func promptGitProviderSelection() (selected string, err error) {
 	for _, provider := range gitProviders {
 		selectableItems = append(selectableItems, ioutils.PromptItem{Option: string(provider), TargetValue: &selected})
 	}
-	println("Choose your project Git provider:")
-	err = ioutils.SelectString(selectableItems, "", func(item ioutils.PromptItem) {
+	log.Output("Choose your project Git provider:")
+	err = ioutils.SelectString(selectableItems, "", false, func(item ioutils.PromptItem) {
 		*item.TargetValue = item.Option
 	})
 	return
@@ -803,8 +823,8 @@ func promptCiProviderSelection() (selected string, err error) {
 	for _, ci := range ciTypes {
 		selectableItems = append(selectableItems, ioutils.PromptItem{Option: string(ci), TargetValue: &selected})
 	}
-	println("Select a CI provider:")
-	err = ioutils.SelectString(selectableItems, "", func(item ioutils.PromptItem) {
+	log.Output("Select a CI provider:")
+	err = ioutils.SelectString(selectableItems, "", false, func(item ioutils.PromptItem) {
 		*item.TargetValue = item.Option
 	})
 	return
@@ -812,7 +832,7 @@ func promptCiProviderSelection() (selected string, err error) {
 
 func (cc *CiSetupCommand) prepareVcsData() (err error) {
 	cc.data.LocalDirPath = DefaultWorkspace
-	for {
+	for i := 1; i < maxRepoCreationAttempts; i++ {
 		err = fileutils.CreateDirIfNotExist(cc.data.LocalDirPath)
 		if err != nil {
 			return err
@@ -824,11 +844,11 @@ func (cc *CiSetupCommand) prepareVcsData() (err error) {
 		if dirEmpty {
 			break
 		} else {
-			log.Error("The '" + cc.data.LocalDirPath + "' directory isn't empty.")
-			ioutils.ScanFromConsole("Choose a name for a directory to be used as the command's workspace", &cc.data.LocalDirPath, "")
-			cc.data.LocalDirPath = clientutils.ReplaceTildeWithUserHome(cc.data.LocalDirPath)
+			cc.data.LocalDirPath = fmt.Sprintf("%s-%d", DefaultWorkspace, i)
 		}
-
+	}
+	if err != nil {
+		return
 	}
 	err = cc.cloneProject()
 	if err != nil {
@@ -845,19 +865,27 @@ func (cc *CiSetupCommand) cloneProject() (err error) {
 		return errorutils.CheckError(err)
 	}
 	cloneOption := &git.CloneOptions{
-		URL:           cc.data.VcsCredentials.Url,
-		Auth:          createCredentials(&cc.data.VcsCredentials),
-		ReferenceName: plumbing.ReferenceName(fmt.Sprintf("refs/heads/%s", cc.data.GitBranch)),
-		// Enable git submodules clone if there any.
+		URL:  cc.data.VcsCredentials.Url,
+		Auth: createCredentials(&cc.data.VcsCredentials),
+		// Enable git submodules clone if their any.
 		RecurseSubmodules: git.DefaultSubmoduleRecursionDepth,
 	}
+	if cc.data.GitBranch != "" {
+		cloneOption.ReferenceName = plumbing.ReferenceName(fmt.Sprintf("refs/heads/%s", cc.data.GitBranch))
+	}
 	err = cc.extractRepositoryName()
 	if err != nil {
 		return
 	}
 	// Clone the given repository to the given directory from the given branch
 	log.Info(fmt.Sprintf("Cloning project %q from: %q into: %q", cc.data.RepositoryName, cc.data.VcsCredentials.Url, cc.data.LocalDirPath))
-	_, err = git.PlainClone(cc.data.LocalDirPath, false, cloneOption)
+	cloneRepo, err := git.PlainClone(cc.data.LocalDirPath, false, cloneOption)
+	if err != nil {
+		return errorutils.CheckError(err)
+	}
+	if cc.data.GitBranch == "" {
+		err = cc.extractDefaultBranchName(cloneRepo)
+	}
 	return errorutils.CheckError(err)
 }
 
@@ -878,7 +906,7 @@ func (cc *CiSetupCommand) stageCiConfigFile(ciFileName string) error {
 func (cc *CiSetupCommand) extractRepositoryName() error {
 	vcsUrl := cc.data.VcsCredentials.Url
 	if vcsUrl == "" {
-		return errorutils.CheckError(errors.New("vcs URL should not be empty"))
+		return errorutils.CheckErrorf("vcs URL should not be empty")
 	}
 	// Trim trailing "/" if one exists
 	vcsUrl = strings.TrimSuffix(vcsUrl, "/")
@@ -887,7 +915,7 @@ func (cc *CiSetupCommand) extractRepositoryName() error {
 	// Split vcs url.
 	splitUrl := strings.Split(vcsUrl, "/")
 	if len(splitUrl) < 3 {
-		return errorutils.CheckError(errors.New("unexpected URL. URL is expected to contain the git provider URL, domain and repository names"))
+		return errorutils.CheckErrorf("unexpected URL. URL is expected to contain the git provider URL, domain and repository names")
 	}
 	cc.data.RepositoryName = strings.TrimSuffix(splitUrl[len(splitUrl)-1], ".git")
 	cc.data.ProjectDomain = splitUrl[len(splitUrl)-2]
@@ -895,22 +923,18 @@ func (cc *CiSetupCommand) extractRepositoryName() error {
 	return nil
 }
 
-func (cc *CiSetupCommand) detectTechnologies() (err error) {
-	indicators := cisetup.GetTechIndicators()
-	filesList, err := fileutils.ListFilesRecursiveWalkIntoDirSymlink(cc.data.LocalDirPath, false)
+func (cc *CiSetupCommand) extractDefaultBranchName(repo *git.Repository) error {
+	headRef, err := repo.Head()
 	if err != nil {
-		return
-	}
-	cc.data.DetectedTechnologies = make(map[cisetup.Technology]bool)
-	for _, file := range filesList {
-		for _, indicator := range indicators {
-			if indicator.Indicates(file) {
-				cc.data.DetectedTechnologies[indicator.GetTechnology()] = true
-				// Same file can't indicate more than one technology.
-				break
-			}
-		}
+		return err
 	}
+	defaultBranch := path.Base(string(headRef.Name())) // refs/heads/branchName > branchName
+	cc.data.GitBranch = defaultBranch
+	return nil
+}
+
+func (cc *CiSetupCommand) detectTechnologies() (err error) {
+	cc.data.DetectedTechnologies, err = coreutils.DetectTechnologies(cc.data.LocalDirPath, true, true)
 	return
 }
 
@@ -941,15 +965,21 @@ func (cc *CiSetupCommand) gitPhase() (err error) {
 		if err != nil {
 			return err
 		}
-		byteToken, err := terminal.ReadPassword(int(syscall.Stdin))
+		//nolint:unconvert
+		byteToken, err := term.ReadPassword(int(syscall.Stdin))
 		if err != nil {
 			log.Error(err)
 			continue
 		}
 		// New-line required after the access token input:
-		fmt.Println()
+		log.Output()
 		cc.data.VcsCredentials.AccessToken = string(byteToken)
-		ioutils.ScanFromConsole("Git branch", &cc.data.GitBranch, cc.defaultData.GitBranch)
+		cc.defaultData.GitBranch = ""
+		gitBranchCaption := "Git branch"
+		if cc.defaultData.GitBranch == "" {
+			gitBranchCaption += " (Leave blank for default)"
+		}
+		ioutils.ScanFromConsole(gitBranchCaption, &cc.data.GitBranch, cc.defaultData.GitBranch)
 		err = cc.prepareVcsData()
 		if err != nil {
 			log.Error(err)
@@ -959,17 +989,16 @@ func (cc *CiSetupCommand) gitPhase() (err error) {
 	}
 }
 
-func (cc *CiSetupCommand) ciProviderPhase() (err error) {
-	var ciType string
+func (cc *CiSetupCommand) ciProviderPhase() {
 	for {
-		ciType, err = promptCiProviderSelection()
+		ciType, err := promptCiProviderSelection()
 		if err != nil {
 			log.Error(err)
 			continue
 		}
 		if ciType == cisetup.Pipelines {
 			// validate that pipelines is available.
-			serviceDetails, err := config.GetSpecificConfig(cisetup.ConfigServerId, false, false)
+			serviceDetails, err := utilsConfig.GetSpecificConfig(cisetup.ConfigServerId, false, false)
 			if err != nil {
 				log.Error(err)
 				continue
@@ -978,7 +1007,6 @@ func (cc *CiSetupCommand) ciProviderPhase() (err error) {
 			pipelinesDetails.AccessToken = ""
 			pipelinesDetails.User = ""
 			pipelinesDetails.Password = ""
-			pipelinesDetails.ApiKey = ""
 
 			pAuth, err := pipelinesDetails.CreatePipelinesAuthConfig()
 			if err != nil {
@@ -1001,18 +1029,18 @@ func (cc *CiSetupCommand) ciProviderPhase() (err error) {
 			_, err = pipelinesMgr.GetSystemInfo()
 			if err == nil {
 				cc.data.CiType = cisetup.CiType(ciType)
-				return nil
+				return
 			}
 			log.Error(err)
 			if _, ok := err.(*pipelinesservices.PipelinesNotAvailableError); ok {
-				err = inactivePipelinesNote()
+				err = inactivePipelinesNote(pipelinesDetails.PipelinesUrl)
 				if err != nil {
 					log.Error(err)
 				}
 			}
 		} else { // The user doesn't choose Pipelines.
 			cc.data.CiType = cisetup.CiType(ciType)
-			return nil
+			return
 		}
 	}
 }
