@@ -7,10 +7,10 @@ import (
 	"errors"
 	"flag"
 	"fmt"
+	"github.com/jfrog/jfrog-client-go/utils/tests"
 	"io"
 	"math/rand"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"regexp"
 	"strconv"
@@ -18,95 +18,96 @@ import (
 	"testing"
 	"time"
 
+	"github.com/urfave/cli"
+
+	buildinfo "github.com/jfrog/build-info-go/entities"
+	"github.com/jfrog/jfrog-cli-core/v2/artifactory/commands/generic"
+	commandutils "github.com/jfrog/jfrog-cli-core/v2/artifactory/commands/utils"
+	artUtils "github.com/jfrog/jfrog-cli-core/v2/artifactory/utils"
+	"github.com/jfrog/jfrog-cli-core/v2/common/spec"
+	commonCliUtils "github.com/jfrog/jfrog-cli-core/v2/common/tests"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/config"
+	"github.com/jfrog/jfrog-cli-core/v2/utils/coreutils"
+	corelog "github.com/jfrog/jfrog-cli-core/v2/utils/log"
+	coreTests "github.com/jfrog/jfrog-cli-core/v2/utils/tests"
 	"github.com/jfrog/jfrog-cli/utils/summary"
-	clientutils "github.com/jfrog/jfrog-client-go/utils"
-
-	"github.com/jfrog/jfrog-client-go/artifactory/buildinfo"
 	"github.com/jfrog/jfrog-client-go/artifactory/services"
-
-	corelog "github.com/jfrog/jfrog-cli-core/utils/log"
-
-	"github.com/jfrog/jfrog-cli-core/artifactory/commands/generic"
-	"github.com/jfrog/jfrog-cli-core/artifactory/spec"
-	artUtils "github.com/jfrog/jfrog-cli-core/artifactory/utils"
-	"github.com/jfrog/jfrog-cli-core/utils/config"
-	"github.com/jfrog/jfrog-cli-core/utils/coreutils"
-	"github.com/stretchr/testify/assert"
-
-	commandutils "github.com/jfrog/jfrog-cli-core/artifactory/commands/utils"
 	"github.com/jfrog/jfrog-client-go/artifactory/services/utils"
 	"github.com/jfrog/jfrog-client-go/auth"
-	"github.com/jfrog/jfrog-client-go/utils/errorutils"
+	clientutils "github.com/jfrog/jfrog-client-go/utils"
 	"github.com/jfrog/jfrog-client-go/utils/io/fileutils"
 	"github.com/jfrog/jfrog-client-go/utils/log"
+	"github.com/stretchr/testify/assert"
 )
 
-var RtUrl *string
-var RtUser *string
-var RtPassword *string
-var RtApiKey *string
-var RtSshKeyPath *string
-var RtSshPassphrase *string
-var RtAccessToken *string
-var RtDistributionUrl *string
-var RtDistributionAccessToken *string
-var BtUser *string
-var BtKey *string
-var BtOrg *string
-var TestArtifactory *bool
-var TestBintray *bool
-var TestArtifactoryProxy *bool
-var TestDistribution *bool
-var TestDocker *bool
-var TestGo *bool
-var TestNpm *bool
-var TestGradle *bool
-var TestMaven *bool
-var DockerRepoDomain *string
-var DockerVirtualRepo *string
-var DockerRemoteRepo *string
-var DockerLocalRepo *string
-var TestNuget *bool
-var HideUnitTestLog *bool
-var TestPip *bool
-var PipVirtualEnv *string
-var TestPlugins *bool
-var timestampAdded bool
-var AccessUrl *string
-var ciRunId *string
+var (
+	JfrogUrl                  *string
+	JfrogUser                 *string
+	JfrogPassword             *string
+	JfrogSshKeyPath           *string
+	JfrogSshPassphrase        *string
+	JfrogAccessToken          *string
+	JfrogTargetUrl            *string
+	JfrogTargetAccessToken    *string
+	JfrogHome                 *string
+	TestArtifactoryProject    *bool
+	TestArtifactory           *bool
+	TestArtifactoryProxy      *bool
+	TestDistribution          *bool
+	TestDocker                *bool
+	TestPodman                *bool
+	TestDockerScan            *bool
+	ContainerRegistry         *string
+	TestGo                    *bool
+	TestNpm                   *bool
+	TestGradle                *bool
+	TestMaven                 *bool
+	TestNuget                 *bool
+	TestPip                   *bool
+	TestPipenv                *bool
+	TestPlugins               *bool
+	TestXray                  *bool
+	TestAccess                *bool
+	TestTransfer              *bool
+	TestLifecycle             *bool
+	HideUnitTestLog           *bool
+	ciRunId                   *string
+	InstallDataTransferPlugin *bool
+	timestampAdded            bool
+)
 
 func init() {
-	RtUrl = flag.String("rt.url", "http://127.0.0.1:8081/artifactory/", "Artifactory url")
-	RtUser = flag.String("rt.user", "admin", "Artifactory username")
-	RtPassword = flag.String("rt.password", "password", "Artifactory password")
-	RtApiKey = flag.String("rt.apikey", "", "Artifactory user API key")
-	RtSshKeyPath = flag.String("rt.sshKeyPath", "", "Ssh key file path")
-	RtSshPassphrase = flag.String("rt.sshPassphrase", "", "Ssh key passphrase")
-	RtAccessToken = flag.String("rt.accessToken", "", "Artifactory access token")
-	RtDistributionUrl = flag.String("rt.distUrl", "", "Distribution url")
-	RtDistributionAccessToken = flag.String("rt.distAccessToken", "", "Distribution access token")
+	JfrogUrl = flag.String("jfrog.url", "http://localhost:8081/", "JFrog platform url")
+	JfrogUser = flag.String("jfrog.user", "admin", "JFrog platform  username")
+	JfrogPassword = flag.String("jfrog.password", "password", "JFrog platform password")
+	JfrogSshKeyPath = flag.String("jfrog.sshKeyPath", "", "Ssh key file path")
+	JfrogSshPassphrase = flag.String("jfrog.sshPassphrase", "", "Ssh key passphrase")
+	JfrogAccessToken = flag.String("jfrog.adminToken", tests.GetLocalArtifactoryTokenIfNeeded(*JfrogUrl), "JFrog platform admin token")
+	JfrogTargetUrl = flag.String("jfrog.targetUrl", "", "JFrog target platform url for transfer tests")
+	JfrogTargetAccessToken = flag.String("jfrog.targetAdminToken", "", "JFrog target platform admin token for transfer tests")
+	JfrogHome = flag.String("jfrog.home", "", "The JFrog home directory of the local Artifactory installation")
 	TestArtifactory = flag.Bool("test.artifactory", false, "Test Artifactory")
+	TestArtifactoryProject = flag.Bool("test.artifactoryProject", false, "Test Artifactory project")
 	TestArtifactoryProxy = flag.Bool("test.artifactoryProxy", false, "Test Artifactory proxy")
-	TestBintray = flag.Bool("test.bintray", false, "Test Bintray")
-	BtUser = flag.String("bt.user", "", "Bintray username")
-	BtKey = flag.String("bt.key", "", "Bintray API Key")
-	BtOrg = flag.String("bt.org", "", "Bintray organization")
 	TestDistribution = flag.Bool("test.distribution", false, "Test distribution")
 	TestDocker = flag.Bool("test.docker", false, "Test Docker build")
+	TestDockerScan = flag.Bool("test.dockerScan", false, "Test Docker scan")
+	TestPodman = flag.Bool("test.podman", false, "Test Podman build")
 	TestGo = flag.Bool("test.go", false, "Test Go")
 	TestNpm = flag.Bool("test.npm", false, "Test Npm")
 	TestGradle = flag.Bool("test.gradle", false, "Test Gradle")
 	TestMaven = flag.Bool("test.maven", false, "Test Maven")
-	DockerRepoDomain = flag.String("rt.dockerRepoDomain", "", "Docker repository domain")
-	DockerVirtualRepo = flag.String("rt.dockerVirtualRepo", "", "Docker virtual repo")
-	DockerRemoteRepo = flag.String("rt.dockerRemoteRepo", "", "Docker remote repo")
-	DockerLocalRepo = flag.String("rt.DockerLocalRepo", "", "Docker local repo")
 	TestNuget = flag.Bool("test.nuget", false, "Test Nuget")
-	HideUnitTestLog = flag.Bool("test.hideUnitTestLog", false, "Hide unit tests logs and print it in a file")
 	TestPip = flag.Bool("test.pip", false, "Test Pip")
-	PipVirtualEnv = flag.String("rt.pipVirtualEnv", "", "Pip virtual-environment path")
+	TestPipenv = flag.Bool("test.pipenv", false, "Test Pipenv")
 	TestPlugins = flag.Bool("test.plugins", false, "Test Plugins")
-	AccessUrl = flag.String("access.url", "", "Access url")
+	TestXray = flag.Bool("test.xray", false, "Test Xray")
+	TestAccess = flag.Bool("test.access", false, "Test Access")
+	TestTransfer = flag.Bool("test.transfer", false, "Test files transfer")
+	TestLifecycle = flag.Bool("test.lifecycle", false, "Test lifecycle")
+	ContainerRegistry = flag.String("test.containerRegistry", "localhost:8082", "Container registry")
+	HideUnitTestLog = flag.Bool("test.hideUnitTestLog", false, "Hide unit tests logs and print it in a file")
+	InstallDataTransferPlugin = flag.Bool("test.installDataTransferPlugin", false, "Install data-transfer plugin on the source Artifactory server")
 	ciRunId = flag.String("ci.runId", "", "A unique identifier used as a suffix to create repositories and builds in the tests")
 }
 
@@ -121,7 +122,10 @@ func removeDirs(dirs ...string) {
 			log.Error(err)
 		}
 		if isExist {
-			os.RemoveAll(dir)
+			err = fileutils.RemoveTempDir(dir)
+			if err != nil {
+				log.Error(errors.New("Cannot remove path: " + dir + " due to: " + err.Error()))
+			}
 		}
 	}
 }
@@ -136,14 +140,14 @@ func VerifyExistLocally(expected, actual []string, t *testing.T) {
 
 func ValidateListsIdentical(expected, actual []string) error {
 	if len(actual) != len(expected) {
-		return errors.New(fmt.Sprintf("Unexpected behavior, \nexpected: [%s], \nfound:    [%s]", strings.Join(expected, ", "), strings.Join(actual, ", ")))
+		return fmt.Errorf("unexpected behavior, \nexpected: [%s], \nfound:    [%s]", strings.Join(expected, ", "), strings.Join(actual, ", "))
 	}
 	err := compare(expected, actual)
 	return err
 }
 
-func ValidateChecksums(filePath string, expectedChecksum fileutils.ChecksumDetails, t *testing.T) {
-	localFileDetails, err := fileutils.GetFileDetails(filePath)
+func ValidateChecksums(filePath string, expectedChecksum buildinfo.Checksum, t *testing.T) {
+	localFileDetails, err := fileutils.GetFileDetails(filePath, true)
 	if err != nil {
 		t.Error("Couldn't calculate sha1, " + err.Error())
 	}
@@ -190,16 +194,6 @@ func GetTestResourcesPath() string {
 	return filepath.ToSlash(dir + "/testdata/")
 }
 
-func GetFilePathForBintray(filename, path string, a ...string) string {
-	for i := 0; i < len(a); i++ {
-		path += a[i] + "/"
-	}
-	if filename != "" {
-		path += filename
-	}
-	return path
-}
-
 func GetFilePathForArtifactory(fileName string) string {
 	return GetTestResourcesPath() + "filespecs/" + fileName
 }
@@ -222,92 +216,6 @@ type PackageSearchResultItem struct {
 	Published bool
 }
 
-type JfrogCli struct {
-	main        func() error
-	prefix      string
-	credentials string
-}
-
-func NewJfrogCli(mainFunc func() error, prefix, credentials string) *JfrogCli {
-	return &JfrogCli{mainFunc, prefix, credentials}
-}
-
-func (cli *JfrogCli) Exec(args ...string) error {
-	spaceSplit := " "
-	os.Args = strings.Split(cli.prefix, spaceSplit)
-	output := strings.Split(cli.prefix, spaceSplit)
-	for _, v := range args {
-		if v == "" {
-			continue
-		}
-		args := strings.Split(v, spaceSplit)
-		os.Args = append(os.Args, v)
-		output = append(output, args...)
-	}
-	if cli.credentials != "" {
-		args := strings.Split(cli.credentials, spaceSplit)
-		os.Args = append(os.Args, args...)
-	}
-
-	log.Info("[Command]", strings.Join(output, " "))
-	return cli.main()
-}
-
-func (cli *JfrogCli) LegacyBuildToolExec(args ...string) error {
-	spaceSplit := " "
-	os.Args = strings.Split(cli.prefix, spaceSplit)
-	os.Args = append(os.Args, args...)
-
-	log.Info("[Command]", os.Args)
-
-	if cli.credentials != "" {
-		args := strings.Split(cli.credentials, spaceSplit)
-		os.Args = append(os.Args, args...)
-	}
-	return cli.main()
-}
-
-func (cli *JfrogCli) WithoutCredentials() *JfrogCli {
-	return &JfrogCli{cli.main, cli.prefix, ""}
-}
-
-type gitManager struct {
-	dotGitPath string
-}
-
-func GitExecutor(dotGitPath string) *gitManager {
-	return &gitManager{dotGitPath: dotGitPath}
-}
-
-func (m *gitManager) GetUrl() (string, string, error) {
-	return m.execGit("config", "--get", "remote.origin.url")
-}
-
-func (m *gitManager) GetRevision() (string, string, error) {
-	return m.execGit("show", "-s", "--format=%H", "HEAD")
-}
-
-func (m *gitManager) GetBranch() (string, string, error) {
-	return m.execGit("branch", "--show-current")
-}
-
-func (m *gitManager) GetMessage(revision string) (string, string, error) {
-	return m.execGit("show", "-s", "--format=%B", revision)
-}
-
-func (m *gitManager) execGit(args ...string) (string, string, error) {
-	var stdout bytes.Buffer
-	var stderr bytes.Buffer
-	cmd := exec.Command("git", args...)
-	cmd.Dir = m.dotGitPath
-	cmd.Stdin = nil
-	cmd.Stdout = &stdout
-	cmd.Stderr = &stderr
-	err := cmd.Run()
-	errorutils.CheckError(err)
-	return strings.TrimSpace(stdout.String()), strings.TrimSpace(stderr.String()), err
-}
-
 func DeleteFiles(deleteSpec *spec.SpecFiles, serverDetails *config.ServerDetails) (successCount, failCount int, err error) {
 	deleteCommand := generic.NewDeleteCommand()
 	deleteCommand.SetThreads(3).SetSpec(deleteSpec).SetServerDetails(serverDetails).SetDryRun(false)
@@ -315,13 +223,18 @@ func DeleteFiles(deleteSpec *spec.SpecFiles, serverDetails *config.ServerDetails
 	if err != nil {
 		return 0, 0, err
 	}
-	defer reader.Close()
+	defer func() {
+		e := reader.Close()
+		if err == nil {
+			err = e
+		}
+	}()
 	return deleteCommand.DeleteFiles(reader)
 }
 
 // This function makes no assertion, caller is responsible to assert as needed.
 func GetBuildInfo(serverDetails *config.ServerDetails, buildName, buildNumber string) (pbi *buildinfo.PublishedBuildInfo, found bool, err error) {
-	servicesManager, err := artUtils.CreateServiceManager(serverDetails, -1, false)
+	servicesManager, err := artUtils.CreateServiceManager(serverDetails, -1, 0, false)
 	if err != nil {
 		return nil, false, err
 	}
@@ -332,24 +245,37 @@ func GetBuildInfo(serverDetails *config.ServerDetails, buildName, buildNumber st
 }
 
 var reposConfigMap = map[*string]string{
-	&DistRepo1:        DistributionRepoConfig1,
-	&DistRepo2:        DistributionRepoConfig2,
-	&DockerRepo:       DockerRepoConfig,
-	&GoRepo:           GoLocalRepositoryConfig,
-	&GradleRepo:       GradleRepositoryConfig,
-	&MvnRepo1:         MavenRepositoryConfig1,
-	&MvnRepo2:         MavenRepositoryConfig2,
-	&MvnRemoteRepo:    MavenRemoteRepositoryConfig,
-	&GradleRemoteRepo: GradleRemoteRepositoryConfig,
-	&NpmRepo:          NpmLocalRepositoryConfig,
-	&NpmRemoteRepo:    NpmRemoteRepositoryConfig,
-	&PypiRemoteRepo:   PypiRemoteRepositoryConfig,
-	&PypiVirtualRepo:  PypiVirtualRepositoryConfig,
-	&RtDebianRepo:     DebianTestRepositoryConfig,
-	&RtLfsRepo:        GitLfsTestRepositoryConfig,
-	&RtRepo1:          Repo1RepositoryConfig,
-	&RtRepo2:          Repo2RepositoryConfig,
-	&RtVirtualRepo:    VirtualRepositoryConfig,
+	&DistRepo1:              DistributionRepoConfig1,
+	&DistRepo2:              DistributionRepoConfig2,
+	&GoRepo:                 GoLocalRepositoryConfig,
+	&GoRemoteRepo:           GoRemoteRepositoryConfig,
+	&GoVirtualRepo:          GoVirtualRepositoryConfig,
+	&GradleRepo:             GradleRepositoryConfig,
+	&MvnRepo1:               MavenRepositoryConfig1,
+	&MvnRepo2:               MavenRepositoryConfig2,
+	&MvnRemoteRepo:          MavenRemoteRepositoryConfig,
+	&GradleRemoteRepo:       GradleRemoteRepositoryConfig,
+	&NpmRepo:                NpmLocalRepositoryConfig,
+	&NpmRemoteRepo:          NpmRemoteRepositoryConfig,
+	&NugetRemoteRepo:        NugetRemoteRepositoryConfig,
+	&YarnRemoteRepo:         YarnRemoteRepositoryConfig,
+	&PypiRemoteRepo:         PypiRemoteRepositoryConfig,
+	&PypiVirtualRepo:        PypiVirtualRepositoryConfig,
+	&PipenvRemoteRepo:       PipenvRemoteRepositoryConfig,
+	&PipenvVirtualRepo:      PipenvVirtualRepositoryConfig,
+	&RtDebianRepo:           DebianTestRepositoryConfig,
+	&RtLfsRepo:              GitLfsTestRepositoryConfig,
+	&RtRepo1:                Repo1RepositoryConfig,
+	&RtRepo2:                Repo2RepositoryConfig,
+	&RtVirtualRepo:          VirtualRepositoryConfig,
+	&TerraformRepo:          TerraformLocalRepositoryConfig,
+	&DockerLocalRepo:        DockerLocalRepositoryConfig,
+	&DockerLocalPromoteRepo: DockerLocalPromoteRepositoryConfig,
+	&DockerRemoteRepo:       DockerRemoteRepositoryConfig,
+	&DockerVirtualRepo:      DockerVirtualRepositoryConfig,
+	&RtDevRepo:              DevRepoRepositoryConfig,
+	&RtProdRepo1:            ProdRepo1RepositoryConfig,
+	&RtProdRepo2:            ProdRepo2RepositoryConfig,
 }
 
 var CreatedNonVirtualRepositories map[*string]string
@@ -382,16 +308,24 @@ func getNeededBuildNames(buildNamesMap map[*bool][]*string) []string {
 // Return local and remote repositories for the test suites, respectfully
 func GetNonVirtualRepositories() map[*string]string {
 	nonVirtualReposMap := map[*bool][]*string{
-		TestArtifactory:  {&RtRepo1, &RtRepo2, &RtLfsRepo, &RtDebianRepo},
-		TestDistribution: {&DistRepo1, &DistRepo2},
-		TestDocker:       {&DockerRepo},
-		TestGo:           {&GoRepo},
-		TestGradle:       {&GradleRepo, &GradleRemoteRepo},
-		TestMaven:        {&MvnRepo1, &MvnRepo2, &MvnRemoteRepo},
-		TestNpm:          {&NpmRepo, &NpmRemoteRepo},
-		TestNuget:        {},
-		TestPip:          {&PypiRemoteRepo},
-		TestPlugins:      {&RtRepo1},
+		TestArtifactory:        {&RtRepo1, &RtRepo2, &RtLfsRepo, &RtDebianRepo, &TerraformRepo},
+		TestArtifactoryProject: {&RtRepo1, &RtRepo2, &RtLfsRepo, &RtDebianRepo},
+		TestDistribution:       {&DistRepo1, &DistRepo2},
+		TestDocker:             {&DockerLocalRepo, &DockerLocalPromoteRepo, &DockerRemoteRepo},
+		TestDockerScan:         {&DockerLocalRepo, &DockerLocalPromoteRepo, &DockerRemoteRepo},
+		TestPodman:             {&DockerLocalRepo, &DockerLocalPromoteRepo, &DockerRemoteRepo},
+		TestGo:                 {&GoRepo, &GoRemoteRepo},
+		TestGradle:             {&GradleRepo, &GradleRemoteRepo},
+		TestMaven:              {&MvnRepo1, &MvnRepo2, &MvnRemoteRepo},
+		TestNpm:                {&NpmRepo, &NpmRemoteRepo},
+		TestNuget:              {&NugetRemoteRepo},
+		TestPip:                {&PypiRemoteRepo},
+		TestPipenv:             {&PipenvRemoteRepo},
+		TestPlugins:            {&RtRepo1},
+		TestXray:               {&NpmRemoteRepo, &NugetRemoteRepo, &YarnRemoteRepo, &GradleRemoteRepo, &MvnRemoteRepo, &GoRepo, &GoRemoteRepo, &PypiRemoteRepo},
+		TestAccess:             {&RtRepo1},
+		TestTransfer:           {&RtRepo1, &RtRepo2, &MvnRepo1, &MvnRemoteRepo, &DockerRemoteRepo},
+		TestLifecycle:          {&RtDevRepo, &RtProdRepo1, &RtProdRepo2},
 	}
 	return getNeededRepositories(nonVirtualReposMap)
 }
@@ -401,13 +335,19 @@ func GetVirtualRepositories() map[*string]string {
 	virtualReposMap := map[*bool][]*string{
 		TestArtifactory:  {&RtVirtualRepo},
 		TestDistribution: {},
-		TestDocker:       {},
-		TestGo:           {},
+		TestDocker:       {&DockerVirtualRepo},
+		TestDockerScan:   {&DockerVirtualRepo},
+		TestPodman:       {&DockerVirtualRepo},
+		TestGo:           {&GoVirtualRepo},
 		TestGradle:       {},
 		TestMaven:        {},
 		TestNpm:          {},
 		TestNuget:        {},
 		TestPip:          {&PypiVirtualRepo},
+		TestPipenv:       {&PipenvVirtualRepo},
+		TestPlugins:      {},
+		TestXray:         {&GoVirtualRepo},
+		TestAccess:       {},
 	}
 	return getNeededRepositories(virtualReposMap)
 }
@@ -432,12 +372,19 @@ func GetBuildNames() []string {
 		TestArtifactory:  {&RtBuildName1, &RtBuildName2, &RtBuildNameWithSpecialChars},
 		TestDistribution: {},
 		TestDocker:       {&DockerBuildName},
+		TestPodman:       {&DockerBuildName},
 		TestGo:           {&GoBuildName},
 		TestGradle:       {&GradleBuildName},
 		TestMaven:        {&MvnBuildName},
 		TestNpm:          {&NpmBuildName, &YarnBuildName},
 		TestNuget:        {&NuGetBuildName},
 		TestPip:          {&PipBuildName},
+		TestPipenv:       {&PipenvBuildName},
+		TestPlugins:      {},
+		TestXray:         {},
+		TestAccess:       {},
+		TestTransfer:     {&MvnBuildName},
+		TestLifecycle:    {&LcBuildName1, &LcBuildName2, &LcBuildName3},
 	}
 	return getNeededBuildNames(buildNamesMap)
 }
@@ -452,8 +399,12 @@ func getSubstitutionMap() map[string]string {
 		"${VIRTUAL_REPO}":              RtVirtualRepo,
 		"${LFS_REPO}":                  RtLfsRepo,
 		"${DEBIAN_REPO}":               RtDebianRepo,
-		"${DOCKER_REPO}":               DockerRepo,
-		"${DOCKER_REPO_DOMAIN}":        *DockerRepoDomain,
+		"${DOCKER_REPO}":               DockerLocalRepo,
+		"${DOCKER_PROMOTE_REPO}":       DockerLocalPromoteRepo,
+		"${DOCKER_REMOTE_REPO}":        DockerRemoteRepo,
+		"${DOCKER_VIRTUAL_REPO}":       DockerVirtualRepo,
+		"${DOCKER_IMAGE_NAME}":         DockerImageName,
+		"${CONTAINER_REGISTRY_DOMAIN}": RtContainerHostName,
 		"${MAVEN_REPO1}":               MvnRepo1,
 		"${MAVEN_REPO2}":               MvnRepo2,
 		"${MAVEN_REMOTE_REPO}":         MvnRemoteRepo,
@@ -461,16 +412,22 @@ func getSubstitutionMap() map[string]string {
 		"${GRADLE_REPO}":               GradleRepo,
 		"${NPM_REPO}":                  NpmRepo,
 		"${NPM_REMOTE_REPO}":           NpmRemoteRepo,
+		"${NUGET_REMOTE_REPO}":         NugetRemoteRepo,
+		"${YARN_REMOTE_REPO}":          YarnRemoteRepo,
 		"${GO_REPO}":                   GoRepo,
-		"${RT_SERVER_ID}":              RtServerId,
-		"${RT_URL}":                    *RtUrl,
-		"${RT_API_KEY}":                *RtApiKey,
-		"${RT_USERNAME}":               *RtUser,
-		"${RT_PASSWORD}":               *RtPassword,
-		"${RT_CREDENTIALS_BASIC_AUTH}": base64.StdEncoding.EncodeToString([]byte(*RtUser + ":" + *RtPassword)),
-		"${RT_ACCESS_TOKEN}":           *RtAccessToken,
+		"${GO_REMOTE_REPO}":            GoRemoteRepo,
+		"${GO_VIRTUAL_REPO}":           GoVirtualRepo,
+		"${TERRAFORM_REPO}":            TerraformRepo,
+		"${SERVER_ID}":                 ServerId,
+		"${URL}":                       *JfrogUrl,
+		"${USERNAME}":                  *JfrogUser,
+		"${PASSWORD}":                  *JfrogPassword,
+		"${RT_CREDENTIALS_BASIC_AUTH}": base64.StdEncoding.EncodeToString([]byte(*JfrogUser + ":" + *JfrogPassword)),
+		"${ACCESS_TOKEN}":              *JfrogAccessToken,
 		"${PYPI_REMOTE_REPO}":          PypiRemoteRepo,
 		"${PYPI_VIRTUAL_REPO}":         PypiVirtualRepo,
+		"${PIPENV_REMOTE_REPO}":        PipenvRemoteRepo,
+		"${PIPENV_VIRTUAL_REPO}":       PipenvVirtualRepo,
 		"${BUILD_NAME1}":               RtBuildName1,
 		"${BUILD_NAME2}":               RtBuildName2,
 		"${BUNDLE_NAME}":               BundleName,
@@ -480,6 +437,14 @@ func getSubstitutionMap() map[string]string {
 		"{PASSWORD_1}":                 Password1,
 		"{USER_NAME_2}":                UserName2,
 		"{PASSWORD_2}":                 Password2,
+		"${LC_BUILD_NAME1}":            LcBuildName1,
+		"${LC_BUILD_NAME2}":            LcBuildName2,
+		"${LC_BUILD_NAME3}":            LcBuildName3,
+		"${RB_NAME1}":                  LcRbName1,
+		"${RB_NAME2}":                  LcRbName2,
+		"${DEV_REPO}":                  RtDevRepo,
+		"${PROD_REPO1}":                RtProdRepo1,
+		"${PROD_REPO2}":                RtProdRepo2,
 	}
 }
 
@@ -489,15 +454,25 @@ func AddTimestampToGlobalVars() {
 	if timestampAdded {
 		return
 	}
-	uniqueSuffix := "-" + strconv.FormatInt(time.Now().Unix(), 10)
+	timestamp := strconv.FormatInt(time.Now().Unix(), 10)
+	uniqueSuffix := "-" + timestamp
 	if *ciRunId != "" {
 		uniqueSuffix = "-" + *ciRunId + uniqueSuffix
 	}
+	// Artifactory accepts only lowercase repository names
+	uniqueSuffix = strings.ToLower(uniqueSuffix)
+
 	// Repositories
-	DockerRepo += uniqueSuffix
 	DistRepo1 += uniqueSuffix
 	DistRepo2 += uniqueSuffix
 	GoRepo += uniqueSuffix
+	GoRemoteRepo += uniqueSuffix
+	GoVirtualRepo += uniqueSuffix
+	DockerLocalRepo += uniqueSuffix
+	DockerLocalPromoteRepo += uniqueSuffix
+	DockerRemoteRepo += uniqueSuffix
+	DockerVirtualRepo += uniqueSuffix
+	TerraformRepo += uniqueSuffix
 	GradleRemoteRepo += uniqueSuffix
 	GradleRepo += uniqueSuffix
 	MvnRemoteRepo += uniqueSuffix
@@ -505,8 +480,12 @@ func AddTimestampToGlobalVars() {
 	MvnRepo2 += uniqueSuffix
 	NpmRepo += uniqueSuffix
 	NpmRemoteRepo += uniqueSuffix
+	NugetRemoteRepo += uniqueSuffix
+	YarnRemoteRepo += uniqueSuffix
 	PypiRemoteRepo += uniqueSuffix
 	PypiVirtualRepo += uniqueSuffix
+	PipenvRemoteRepo += uniqueSuffix
+	PipenvVirtualRepo += uniqueSuffix
 	RtDebianRepo += uniqueSuffix
 	RtLfsRepo += uniqueSuffix
 	RtRepo1 += uniqueSuffix
@@ -514,6 +493,9 @@ func AddTimestampToGlobalVars() {
 	RtRepo1And2Placeholder += uniqueSuffix
 	RtRepo2 += uniqueSuffix
 	RtVirtualRepo += uniqueSuffix
+	RtDevRepo += uniqueSuffix
+	RtProdRepo1 += uniqueSuffix
+	RtProdRepo2 += uniqueSuffix
 
 	// Builds/bundles/images
 	BundleName += uniqueSuffix
@@ -522,54 +504,42 @@ func AddTimestampToGlobalVars() {
 	DotnetBuildName += uniqueSuffix
 	GoBuildName += uniqueSuffix
 	GradleBuildName += uniqueSuffix
-	MvnBuildName += uniqueSuffix
 	NpmBuildName += uniqueSuffix
 	YarnBuildName += uniqueSuffix
+	MvnBuildName += uniqueSuffix
 	NuGetBuildName += uniqueSuffix
 	PipBuildName += uniqueSuffix
+	PipenvBuildName += uniqueSuffix
 	RtBuildName1 += uniqueSuffix
 	RtBuildName2 += uniqueSuffix
 	RtBuildNameWithSpecialChars += uniqueSuffix
 	RtPermissionTargetName += uniqueSuffix
+	LcBuildName1 += uniqueSuffix
+	LcBuildName2 += uniqueSuffix
+	LcBuildName3 += uniqueSuffix
+	LcRbName1 += uniqueSuffix
+	LcRbName2 += uniqueSuffix
+	LcRbName3 += uniqueSuffix
 
 	// Users
 	UserName1 += uniqueSuffix
 	UserName2 += uniqueSuffix
-	rand.Seed(time.Now().Unix())
-	Password1 += uniqueSuffix + strconv.FormatFloat(rand.Float64(), 'f', 2, 32)
-	Password2 += uniqueSuffix + strconv.FormatFloat(rand.Float64(), 'f', 2, 32)
+
+	randomSequence := rand.New(rand.NewSource(time.Now().Unix()))
+	Password1 += uniqueSuffix + strconv.FormatFloat(randomSequence.Float64(), 'f', 2, 32)
+	Password2 += uniqueSuffix + strconv.FormatFloat(randomSequence.Float64(), 'f', 2, 32)
+
+	// Projects
+	ProjectKey += timestamp[len(timestamp)-7:]
 
 	timestampAdded = true
 }
 
+// Replace all variables in the form of ${VARIABLE} in the input file, according to the substitution map (see getSubstitutionMap()).
+// path - Path to the input file.
+// destPath - Path to the output file. If empty, the output file will be under ${CWD}/tmp/.
 func ReplaceTemplateVariables(path, destPath string) (string, error) {
-	content, err := os.ReadFile(path)
-	if err != nil {
-		return "", errorutils.CheckError(err)
-	}
-
-	for name, value := range getSubstitutionMap() {
-		content = bytes.Replace(content, []byte(name), []byte(value), -1)
-	}
-	if destPath == "" {
-		destPath, err = os.Getwd()
-		if err != nil {
-			return "", errorutils.CheckError(err)
-		}
-		destPath = filepath.Join(destPath, Temp)
-	}
-	err = os.MkdirAll(destPath, 0700)
-	if err != nil {
-		return "", errorutils.CheckError(err)
-	}
-	specPath := filepath.Join(destPath, filepath.Base(path))
-	log.Info("Creating spec file at:", specPath)
-	err = os.WriteFile(specPath, []byte(content), 0700)
-	if err != nil {
-		return "", errorutils.CheckError(err)
-	}
-
-	return specPath, nil
+	return commonCliUtils.ReplaceTemplateVariables(path, destPath, getSubstitutionMap())
 }
 
 func CreateSpec(fileName string) (string, error) {
@@ -588,14 +558,11 @@ func ConvertSliceToMap(props []utils.Property) map[string][]string {
 
 // Set user and password from access token.
 // Return the original user and password to allow restoring them in the end of the test.
-func SetBasicAuthFromAccessToken(t *testing.T) (string, string) {
-	var err error
-	origUser := *RtUser
-	origPassword := *RtPassword
-
-	*RtUser, err = auth.ExtractUsernameFromAccessToken(*RtAccessToken)
-	assert.NoError(t, err)
-	*RtPassword = *RtAccessToken
+func SetBasicAuthFromAccessToken() (string, string) {
+	origUser := *JfrogUser
+	origPassword := *JfrogPassword
+	*JfrogUser = auth.ExtractUsernameFromAccessToken(*JfrogAccessToken)
+	*JfrogPassword = *JfrogAccessToken
 	return origUser, origPassword
 }
 
@@ -611,7 +578,7 @@ func CleanUpOldItems(baseItemNames []string, getActualItems func() ([]string, er
 	}
 	now := time.Now()
 	for _, baseItemName := range baseItemNames {
-		itemPattern := regexp.MustCompile(`^` + baseItemName + `-(\d*)$`)
+		itemPattern := regexp.MustCompile(`^` + baseItemName + `[\w-]*-(\d*)$`)
 		for _, item := range actualItems {
 			regexGroups := itemPattern.FindStringSubmatch(item)
 			if regexGroups == nil {
@@ -621,7 +588,7 @@ func CleanUpOldItems(baseItemNames []string, getActualItems func() ([]string, er
 
 			itemTimestamp, err := strconv.ParseInt(regexGroups[len(regexGroups)-1], 10, 64)
 			if err != nil {
-				log.Warn("Error while parsing timestamp of ", item, err)
+				log.Warn("Error while parsing timestamp of", item, err)
 				continue
 			}
 
@@ -633,28 +600,37 @@ func CleanUpOldItems(baseItemNames []string, getActualItems func() ([]string, er
 	}
 }
 
-// Set new logger with output redirection to a buffer.
-// Caller is responsible to set the old log back.
-func RedirectLogOutputToBuffer() (buffer *bytes.Buffer, previousLog log.Log) {
-	previousLog = log.Logger
-	newLog := log.NewLogger(corelog.GetCliLogLevel(), nil)
-	buffer = &bytes.Buffer{}
-	newLog.SetOutputWriter(buffer)
-	log.SetLogger(newLog)
-	return buffer, previousLog
-}
-
 // Set new logger with output redirection to a null logger. This is useful for negative tests.
 // Caller is responsible to set the old log back.
 func RedirectLogOutputToNil() (previousLog log.Log) {
 	previousLog = log.Logger
 	newLog := log.NewLogger(corelog.GetCliLogLevel(), nil)
 	newLog.SetOutputWriter(io.Discard)
-	newLog.SetLogsWriter(io.Discard)
+	newLog.SetLogsWriter(io.Discard, 0)
 	log.SetLogger(newLog)
 	return previousLog
 }
 
+// Redirect output to a file, execute the command and read output.
+// The reason for redirecting to a file and not to a buffer is the limited
+// size of the buffer while using os.Pipe.
+func GetCmdOutput(t *testing.T, jfrogCli *coreTests.JfrogCli, cmd ...string) ([]byte, error) {
+	oldStdout := os.Stdout
+	temp, err := os.CreateTemp("", "output")
+	assert.NoError(t, err)
+	os.Stdout = temp
+	defer func() {
+		os.Stdout = oldStdout
+		assert.NoError(t, temp.Close())
+		assert.NoError(t, os.Remove(temp.Name()))
+	}()
+	err = jfrogCli.Exec(cmd...)
+	assert.NoError(t, err)
+	content, err := os.ReadFile(temp.Name())
+	assert.NoError(t, err)
+	return content, err
+}
+
 func VerifySha256DetailedSummaryFromBuffer(t *testing.T, buffer *bytes.Buffer, logger log.Log) {
 	content := buffer.Bytes()
 	buffer.Reset()
@@ -676,11 +652,44 @@ func VerifySha256DetailedSummaryFromBuffer(t *testing.T, buffer *bytes.Buffer, l
 }
 
 func VerifySha256DetailedSummaryFromResult(t *testing.T, result *commandutils.Result) {
-	result.Reader()
-	reader := result.Reader()
-	defer reader.Close()
-	assert.NoError(t, reader.GetError())
-	for transferDetails := new(clientutils.FileTransferDetails); reader.NextRecord(transferDetails) == nil; transferDetails = new(clientutils.FileTransferDetails) {
-		assert.Equal(t, 64, len(transferDetails.Sha256), "Summary validation failed - invalid sha256 has returned from artifactory")
+	if assert.NotNil(t, result) {
+		reader := result.Reader()
+		defer func() {
+			assert.NoError(t, reader.Close())
+		}()
+		assert.NoError(t, reader.GetError())
+		for transferDetails := new(clientutils.FileTransferDetails); reader.NextRecord(transferDetails) == nil; transferDetails = new(clientutils.FileTransferDetails) {
+			assert.Equal(t, 64, len(transferDetails.Sha256), "Summary validation failed - invalid sha256 has returned from artifactory")
+		}
 	}
 }
+
+func SkipKnownFailingTest(t *testing.T) {
+	skipDate := time.Date(2023, time.November, 1, 0, 0, 0, 0, time.UTC)
+	if time.Now().Before(skipDate) {
+		t.Skip("Skipping a known failing test, will resume testing after ", skipDate.String())
+	} else {
+		t.Error("Not skipping test. Please fix the test or delay the skipMonth")
+	}
+}
+
+func CreateContext(t *testing.T, testFlags, testArgs []string) (*cli.Context, *bytes.Buffer) {
+	flagSet := createFlagSet(t, testFlags, testArgs)
+	app := cli.NewApp()
+	app.Writer = &bytes.Buffer{}
+	return cli.NewContext(app, flagSet, nil), &bytes.Buffer{}
+}
+
+// Create flag set with input flags and arguments.
+func createFlagSet(t *testing.T, flags []string, args []string) *flag.FlagSet {
+	flagSet := flag.NewFlagSet("TestFlagSet", flag.ContinueOnError)
+	flags = append(flags, "url=http://127.0.0.1:8081/artifactory")
+	var cmdFlags []string
+	for _, curFlag := range flags {
+		flagSet.String(strings.Split(curFlag, "=")[0], "", "")
+		cmdFlags = append(cmdFlags, "--"+curFlag)
+	}
+	cmdFlags = append(cmdFlags, args...)
+	assert.NoError(t, flagSet.Parse(cmdFlags))
+	return flagSet
+}
